<?php
 
namespace App\Exceptions;
 
use Exception;
use Illuminate\Validation\ValidationException;
use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Laravel\Lumen\Exceptions\Handler as ExceptionHandler;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Log;
use Illuminate\Http\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\MethodNotAllowedHttpException;
use Illuminate\Http\Exception\HttpResponseException;
use Symfony\Component\Debug\Exception\FatalThrowableError;
use Firebase\JWT\JWT;
 
class SnowException extends ExceptionHandler
{
    private $respuesta;
    /**
     * Report the exception.
     *
     * @return void
     */
    public function report()
    {
    }
 
    /**
     * Render the exception into an HTTP response.
     *
     * @param  \Illuminate\Http\Request
     * @return \Illuminate\Http\Response
     */
    public function render($request, Exception $e)
    {
        $this->respuesta = new \stdclass();
        $this->respuesta->notificacion = new \stdclass();
        $this->respuesta->notificacion->estado = -1;
        $this->respuesta->notificacion->mensaje = '';
        $this->respuesta->data = new \stdclass();

        
        Log::info('---------ERROR CONTROLADO-------');
        /*
        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);
        Log::info('Usuario sistema: ' . $usuarioSistema);
        */

        if ($e instanceof HttpResponseException) {
            $response['status']   = Response::HTTP_INTERNAL_SERVER_ERROR;
        } elseif ($e instanceof MethodNotAllowedHttpException) {
            $response['status'] = Response::HTTP_METHOD_NOT_ALLOWED;
        } elseif ($e instanceof NotFoundHttpException) {
            $response['status'] = Response::HTTP_NOT_FOUND;
        } elseif ($e instanceof AuthorizationException) {
            $response['status'] = Response::HTTP_FORBIDDEN;
            $e      = new AuthorizationException('HTTP_FORBIDDEN', $response['status']);
        } elseif ($e instanceof ValidationException && $e->getResponse()) {
            $response['status']   = Response::HTTP_BAD_REQUEST;
            $e        = new ValidationException('HTTP_BAD_REQUEST', $response['status'], $e);
        } elseif ($e instanceof ValidationException) {
            $response['status']   = 422;
            $response['errors'] = $e->validator->errors();
        } elseif ($e instanceof ModelNotFoundException) {
          $response['status'] = 404;
        } elseif ($e instanceof UnableToExecuteRequestException) {
            $response['status'] = $e->getCode();
        } elseif ($e instanceof FatalThrowableError) {
            $response['status'] = Response::HTTP_INTERNAL_SERVER_ERROR;
        }else{
            $response['status'] = Response::HTTP_BAD_REQUEST;
            $response['unhandled'] = 'no hay';
        }

        if ($request->isMethod('post')) {
            $input = $request->all();
            Log::info('POST = ' . json_encode($input));
        }else{
            $url = $request->fullUrl();
            Log::info('GET = ' . json_encode($url));   
        }

        $this->respuesta->data = $response;
        if ($response['status']) {
            $response['message'] = $e->getMessage();
            $response['error_code'] = $e->getCode() ?? '';
            // $response['exception'] = $e->getTraceAsString() ?? '';
            if (app()->environment() == 'local'){
                $response['file'] = $e->getFile() ?? '';
                $response['line'] = $e->getLine() ?? '';
            }
            Log::error(json_encode($this->respuesta));
            return response()->json($this->respuesta, $response['status']);
        } else {
            return parent::render($request, $e);
        }
                
    }
}