<?php

namespace App\Http\Controllers\Caja;

use App\Models\Caja\TemporalVenta;
use App\Http\Controllers\Controller as Controller;
use Illuminate\Http\Request as Peticion;
use Firebase\JWT\JWT;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;

class TemporalVentaController extends Controller
{
    private $respuesta;
    public function __construct()
    {
        //
        $this->respuesta = new \stdclass();
        $this->respuesta->notificacion = new \stdclass();
        $this->respuesta->notificacion->mensaje = '';
        $this->respuesta->data = new \stdclass();
    }

    public function show(Peticion $request)
    {
        $per_page = 105;
        $params = [];
        $sorting = 'nombre';
        $order = 'desc';
        if ($request->has('per_page')) {
            $per_page = $request->per_page;
        }
        if ($request->has('order')) {
            $order = $request->order;
        }
        if ($request->has('sorting')) {
            $sorting = $request->sorting;
        }

        if (!is_null($request->getQueryString())) {
            $params = [];
            if ($request->has('cliente_nombre')) {
                $params[] = ['nomcliente_nombrebre', 'ilike', "%".$request->cliente_nombre."%"];
            }
            $data = TemporalVenta::where($params)
            ->orderBy($sorting, $order)
            ->paginate($per_page);
            $this->respuesta->notificacion->mensaje = 'Exito';
            $this->respuesta->notificacion->tipo = 'success';
            $this->respuesta->data = $data;

        }else{
            $data = TemporalVenta::orderBy($sorting, $order)
            ->paginate($per_page);
        }

        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );

    }

    public function find($id)
    {
        $data = TemporalVenta::findOrFail($id);
        if(!$data){
            $this->respuesta->notificacion->mensaje = 'No se encontro informacion';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                200
            );
        }
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function store(Peticion $request)
    {
        $newRow = new TemporalVenta;
        $this->recuperarDatos($request, $newRow);
        $newRow->save();
        //formamos la respuesta para respuesta
        $data = new \stdClass;
        $data->id = $newRow->id;
        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function update($id, Peticion $request)
    {        
        //verificar si ya existe el registro
        $ventaEnProceso = TemporalVenta::find($id);
        
        //si ya existe el crédito
        $ventaActualizar = $ventaEnProceso;

        $this->recuperarDatos($request, $ventaActualizar);

        //verificar si cambio la etapa
        $ventaActualizar->save();

        //formamos la respuesta para respuesta
        $data = new \stdClass;
        $data->id = $newRow->id;
        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );


    }

    public function delete($id)
    {
        $cliente = Articulo::findOrFail($id);
        $result = $cliente->delete();

        $data = new \stdClass;
        $data->id = $newRow->id;
        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $result;
        return response()->json($this->respuesta,
            200
        );
    }

    private function recuperarDatos(Peticion $request, &$registroGuardar){

        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);
        //usuario que captura el registro
        $registroGuardar->cajero_id = $usuarioSistema->data->id;

        if ($request->has('status')) {
            $registroGuardar->status = $request->status;
        }
        if ($request->has('subtotal')) {
            $registroGuardar->subtotal = $request->subtotal;
        }
        if ($request->has('descuento')) {
            $registroGuardar->descuento = $request->descuento;
        }
        if ($request->has('iva')) {
            $registroGuardar->iva = $request->iva;
        }
        if ($request->has('total')) {
            $registroGuardar->total = $request->total;
        }
        if ($request->has('fechatienda')) {
            $registroGuardar->fechatienda = $request->fechatienda;
        }
        if ($request->has('sucursal_id')) {
            $registroGuardar->sucursal_id = $request->sucursal_id;
        }
        if ($request->has('condicionventa')) {
            $registroGuardar->condicionventa = $request->condicionventa;
        }
        if ($request->has('enganche')) {
            $registroGuardar->enganche = $request->enganche;
        }
        if ($request->has('saldopendiente')) {
            $registroGuardar->saldopendiente = $request->saldopendiente;
        }
        if ($request->has('vendedor_id')) {
            $registroGuardar->vendedor_id = $request->vendedor_id;
        }
        if ($request->has('vendedor_nombre')) {
            $registroGuardar->vendedor_nombre = $request->vendedor_nombre;
        }
        if ($request->has('cliente_id')) {
            $registroGuardar->cliente_id = $request->cliente_id;
        }
        if ($request->has('cliente_nombre')) {
            $registroGuardar->cliente_nombre = $request->cliente_nombre;
        }
        if ($request->has('venta_detalles')) {
            $registroGuardar->venta_detalles = $request->venta_detalles;
        }
        if ($request->has('venta_pagos')) {
            $registroGuardar->venta_pagos = $request->venta_pagos;
        }

    }
}
