<?php

namespace App\Http\Controllers\Caja;

use App\Models\Caja\Venta;
use App\Models\Caja\VentaDetalle;
use App\Models\Caja\MovimientoPago;
use App\Models\Inventario\Inventario;
use App\Models\Inventario\MovimientoArticulo;
use App\Models\Catalogos\ArticuloPaquete;
use App\Models\Configuracion\Configuracion;
use App\Models\Caja\RecargaTiempoAire;
use App\Models\Catalogos\Cliente;
use App\Models\Credito\MovimientoCredito;
use App\Models\Credito\Cartera;
use App\Models\Credito\ControlCreditoCliente;
use App\Http\Controllers\Controller as Controller;
use Illuminate\Http\Request as Peticion;
use Firebase\JWT\JWT;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use GuzzleHttp\Client;
use Log;

class VentaController extends Controller
{
    private $respuesta;
    private $servicio;
    public function __construct()
    {
        //
        $this->servicio = getenv('API_TA_URL');
        $this->respuesta = new \stdclass();
        $this->respuesta->notificacion = new \stdclass();
        $this->respuesta->notificacion->mensaje = '';
        $this->respuesta->data = new \stdclass();
    }

    public function show(Peticion $request)
    {
        $per_page = 105;
        $params = [];
        $sorting = 'nombre';
        $order = 'desc';
        if ($request->has('per_page')) {
            $per_page = $request->per_page;
        }
        if ($request->has('order')) {
            $order = $request->order;
        }
        if ($request->has('sorting')) {
            $sorting = $request->sorting;
        }

        if (!is_null($request->getQueryString())) {
            $params = [];
            if ($request->has('codigoarticulo')) {
                $params[] = ['codigoarticulo', '=', $request->codigoarticulo];
                $data =  Articulo::where($params)
                ->with('categoria')
                ->with('unidadmedida')
                ->with('tipoetiqueta')
                //->select('id','numcred','nombre','omisos','estatus','status','procesoactual')
                ->first();
                $this->respuesta->notificacion->mensaje = 'Exito';
                $this->respuesta->notificacion->tipo = 'success';
                $this->respuesta->data = $data;
            }else{
                if ($request->has('nombre')) {
                    $params[] = ['nombre', 'ilike', "%".$request->nombre."%"];
                }
                $data = Articulo::where($params)
                ->with('categoria')
                ->with('unidadmedida')
                ->with('tipoetiqueta')
                ->orderBy($sorting, $order)
                ->paginate($per_page);

                $this->respuesta->notificacion->mensaje = 'Exito';
                $this->respuesta->notificacion->tipo = 'success';
                $this->respuesta->data = $data;
            }

        }else{
            $data = Articulo::with('categoria')
            ->with('unidadmedida')
            ->with('tipoetiqueta')
            ->orderBy($sorting, $order)
            ->paginate($per_page);
        }

        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;

        return response()->json($this->respuesta,
            200
        );

    }

    public function find($id)
    {
        $data = Articulo::
        with('categoria')
        ->with('unidadmedida')
        ->with('tipoetiqueta')
            ->findOrFail($id);
        if(!$data){
            $this->respuesta->notificacion->mensaje = 'No se encontro informacion';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                200
            );
        }
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function store(Peticion $request)
    {
        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);
        $response = 0;
        $detalles = array();
        $pagos = array();
        $row = new Venta;

        // recuperamos los datos de la venta que llega en la petición
        $this->recuperarDatos($request, $row, $detalles, $pagos);

        if ( $request->tipopago_id == 1){
            // si es venta de producto
            $respuestaVenta = $this->registrarVenta($request, $row, $detalles, $pagos, $usuarioSistema);

            if( $respuestaVenta->notificacion->estado == 0){
                $this->respuesta->notificacion->mensaje = $respuestaVenta->notificacion->mensaje;
                $this->respuesta->notificacion->tipo = $respuestaVenta->notificacion->tipo;
                $this->respuesta->data = $respuestaVenta->data;
                return response()->json($this->respuesta,
                    200
                );
            }else{
                $this->respuesta->notificacion->mensaje = $respuestaVenta->notificacion->mensaje;
                $this->respuesta->notificacion->tipo = $respuestaVenta->notificacion->tipo;
                $this->respuesta->data = $respuestaVenta->data;
                return response()->json($this->respuesta,
                    $respuestaVenta->notificacion->estado
                );
            }


        }else if ( $request->tipopago_id == 3 ){
            // si es venta de tiempo aire
            $respuestaVenta = $this->registrarRecargaTA($request, $row, $detalles, $pagos, $usuarioSistema);

            if( $respuestaVenta->notificacion->estado == 0){
                $this->respuesta->notificacion->mensaje = $respuestaVenta->notificacion->mensaje;
                $this->respuesta->notificacion->tipo = $respuestaVenta->notificacion->tipo;
                $this->respuesta->data = $respuestaVenta->data;
                return response()->json($this->respuesta,
                    200
                );
            }else{
                $this->respuesta->notificacion->mensaje = $respuestaVenta->notificacion->mensaje;
                $this->respuesta->notificacion->tipo = $respuestaVenta->notificacion->tipo;
                $this->respuesta->data = $respuestaVenta->data;
                return response()->json($this->respuesta,
                    $respuestaVenta->notificacion->estado
                );
            }

        }else if ( $request->tipopago_id == 4 ){
            // si es pago de servicios
        }

    }

    public function update($id, Peticion $request)
    {
        $inventario = new InventarioController;
        
        //verificar si ya existe el registro
        $creditoBuscado = CreditoJudicial::find($id);
        
        //si ya existe el crédito
        $creditoActualizar = $creditoBuscado;

        $this->recuperarDatos($request, $creditoActualizar);

        //verificar si cambio la etapa
        $creditoActualizar->save();

        //Guardamos el inventario del articulo
        $datos = new \stdclass();
        $inventario->actualizarInventario($request, $datos);

        //formamos la respuesta para respuesta
        $data = new \stdClass;
        $data->id = $newRow->id;
        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );


    }

    public function delete($id)
    {
        $cliente = Articulo::findOrFail($id);
        $result = $cliente->delete();

        $data = new \stdClass;
        $data->id = $newRow->id;
        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $result;
        return response()->json($this->respuesta,
            200
        );
    }

    private function recuperarDatos(Peticion $request, &$row, &$detalles, &$pagos){

        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);

        $row->cajero_id = $usuarioSistema->data->id;

        if ($request->has('subtotal')) {
            $row->subtotal = $request->subtotal;
        }
        if ($request->has('descuento')) {
            $row->descuento = $request->descuento;
        }
        if ($request->has('iva')) {
            $row->iva = $request->iva;
        }
        if ($request->has('total')) {
            $row->total = $request->total;
        }
        if ($request->has('status')) {
            $row->status = $request->status;
        }
        if ($request->has('fechatienda')) {
            $row->fechatienda = $request->fechatienda;
        }
        if ($request->has('sucursal_id')) {
            $row->sucursal_id = $request->sucursal_id;
        }
        if ($request->has('vendedor_id')) {
            $row->vendedor_id = $request->vendedor_id;
        }
        if ($request->has('cliente_id')) {
            $row->cliente_id = $request->cliente_id;
        }
        if ($request->has('condicionventa_id')) {
            $row->condicionventa_id = $request->condicionventa_id;
        }
        if ($request->has('enganche')) {
            $row->enganche = $request->enganche;
        }
        if ($request->has('_uuid')) {
            $row->_uuid = $request->_uuid;
        }
        if ($request->has('detalles')) {
            $detalles = json_decode(json_encode($request->detalles));
        }else{
            $this->respuesta->notificacion->estado = 1;
            $this->respuesta->notificacion->mensaje = 'No hay ningun articulo registrado en la venta';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = null;
            return response()->json($this->respuesta,
                400
            );
        }

        //registramos las formas de pago
        if ($request->has('formaspago')) {
            $pagos = json_decode(json_encode($request->formaspago));
        }else{
            $this->respuesta->notificacion->estado = 1;
            $this->respuesta->notificacion->mensaje = 'Debe registrar las formas de pago primero';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = null;
            return response()->json($this->respuesta,
                400
            );
        }

    }

    private function registrarVenta(Peticion $request, $row, $detalles, $pagos, $usuarioSistema){

        $response = new \stdclass();
        $response->notificacion = new \stdclass();
        $response->notificacion->estado = 0;
        $response->notificacion->mensaje = 'Exito';
        $response->notificacion->tipo = 'success';
        $response->data = new \stdclass();


        Log::info('REquest = ' . $request->getContent());
        DB::transaction(function() use ($request, $usuarioSistema, $row, $detalles, $pagos, &$response) {
            $fechaHoy = date('Y-m-d H:i:s');
            $datosCliente = null;
            $pos = 0;
            try {
                //grabamos la venta
                $row->save();

                // obtenemos el detalle de la venta
                foreach ($detalles as $item) {
                    $pos= $pos +1;
                    $detalle = new VentaDetalle;
                    $detalle->venta_id = $row->id; // se guarda el id generado automaticamente en la tabla ventas
                    $detalle->codigoarticulo = $item->codigoarticulo;
                    $detalle->tipoproducto = $item->tipoproducto;
                    $detalle->cantidad = $item->cantidad;
                    $detalle->preciounitario = $item->preciounitario;
                    $detalle->descuento = $item->descuento;
                    $detalle->iva = $item->iva;
                    $detalle->total = $item->total;
                    $detalle->posicion =  $pos;
                    $detalle->costo = $item->costo;
                    $detalle->status = $item->status;
                    $detalle->save();
                    
                    if ( $item->tipoproducto == 1){
                        // actualizamos inventario
                        $inventario = Inventario::
                            where('sucursal_id', '=', $request->sucursal_id)
                            ->where('codigoarticulo', '=', $item->codigoarticulo)
                            ->first();
                        $inventario->existencia = $inventario->existencia - 1;
                        $inventario->save();
                    }

                    //registramos kardex de articulo
                    $movimiento = new MovimientoArticulo;
                    $movimiento->sucursal_id = $request->sucursal_id; 
                    $movimiento->referencia_id = $row->id; // se guarda el id generado automaticamente en la tabla ventas
                    $movimiento->tipomovimiento_id = 1; //venta, 
                    $movimiento->codigoarticulo = $item->codigoarticulo;
                    $movimiento->cantidad = $item->cantidad;
                    $movimiento->precioventa = $item->preciounitario;
                    $movimiento->preciocosto = $item->costo;
                    $movimiento->usuario_id = $usuarioSistema->data->id;
                    $movimiento->descuento =  $item->descuento;
                    $movimiento->importe = $item->total;
                    $movimiento->save();

                    //verificamos si es paquete para obtener los codigos del paquete y afectar inventario
                    if( $item->espaquete == 1 ){
                        Log::info(' $item->espaquete = ' . $item->espaquete);
                        //obtener los codigos del paquete
                        $articulos = ArticuloPaquete::where('articulo_id', '=', $item->codigoarticulo)
                                ->get();
                        foreach ($articulos as $itemPaquete) {
                            // actualizamos inventario
                            $inventario = Inventario::
                                where('sucursal_id', '=', $request->sucursal_id)
                                ->where('codigoarticulo', '=', $itemPaquete->codigoarticulo)
                                ->first();
                            $inventario->existencia = $inventario->existencia - $itemPaquete->cantidad;
                            $inventario->save();
                            //actualizar inventario y movimiento   
                            $movimiento2 = new MovimientoArticulo;
                            $movimiento2->sucursal_id = $request->sucursal_id; 
                            $movimiento2->referencia_id = $row->id; // se guarda el id generado automaticamente en la tabla ventas
                            $movimiento2->tipomovimiento_id = 37; //venta por paquete, 
                            $movimiento2->codigoarticulo = $itemPaquete->codigoarticulo;
                            $movimiento2->cantidad = $itemPaquete->cantidad;
                            $movimiento2->precioventa = $inventario->preciodeventa;
                            $movimiento2->preciocosto = $inventario->preciocosto;
                            $movimiento2->usuario_id = $usuarioSistema->data->id;
                            $movimiento2->descuento =  0;
                            $movimiento2->importe = $itemPaquete->cantidad * $inventario->preciodeventa;
                            $movimiento2->save();                         
                        }

                    }

                    //verificamos si tiene control de venta por lote o fecha caducidad
                        //obtener existencia de mae_inventarioslotes
                        // si hay existncia mayor o igual a la cantida de venta seactualiza mae_inventarioslotes
                        // se quita el registro de mae_inventarioslotes cuando no hay mas existencia para ese lote

                }

                foreach ($pagos as $item) {
                    $pagos = new MovimientoPago;
                    $pagos->referencia_id = $row->id; // se guarda el id generado automaticamente en la tabla ventas
                    $pagos->origen_id = $item->origen_id;
                    $pagos->formapago_id = $item->formapago_id;
                    $pagos->importe = $item->importe;
                    $pagos->importeentrega = $item->importeentrega;
                    $pagos->save();

                    //si es forma de pago con tarjeta y la venta no requiere factura guardar la comision cobrada
                    //mov_formasdepagocomision

                }

                //si es a credito
                if( $request->condicionventa_id == 2){
                    // obtenemos el limite de credito y validamos si tiene margen
                    if($request->has('cliente_id')){

                        $datosCliente = Cliente::where('id','=',$request->cliente_id)
                        ->with('lineacredito')
                        ->first();

                    }else{
                        throw new \Exception('Debe seleccionar un cliente para venta a credito',400);
                    }

                    if( !$datosCliente ){
                        throw new \Exception('Cliente no encontrado',400);
                    }
                    
                    //obtenemos el saldo actual de la cartera del cliente
                    $saldoActual = DB::table('mov_cartera')
                                ->where('cliente_id', '=', $request->cliente_id)
                                ->sum('saldopendiente');
                    
                    //calaculamos el nuevo saldo
                    $nuevoSaldo = $saldoActual  + $request->saldopendiente;

                    if($datosCliente->lineacredito->limitedecredito < $nuevoSaldo){
                        throw new \Exception('Cliente excede su limite de crédito',400);
                    }
                    //guardamos el movimiento del cliente en mov_creditomovimientos VENTA A CREDITO
                    $movimientoCredito = new MovimientoCredito;
                    $movimientoCredito->sucursal_id = $request->sucursal_id;
                    $movimientoCredito->cliente_id = $request->cliente_id;
                    $movimientoCredito->referencia = $row->id;
                    $movimientoCredito->tipomovimiento_id = 2;
                    $movimientoCredito->importemovimiento = $request->total;
                    $movimientoCredito->usuario_id = $usuarioSistema->data->id;
                    $movimientoCredito->save();
                    
                    // SI HAY ENGANCHE guardamos el movimiento del cliente en mov_creditomovimientos
                    if ( $request->enganche > 0 ){
                        $movimientoCredito2 = new MovimientoCredito;
                        $movimientoCredito2->sucursal_id = $request->sucursal_id;
                        $movimientoCredito2->cliente_id = $request->cliente_id;
                        $movimientoCredito2->referencia = $row->id;
                        $movimientoCredito2->tipomovimiento_id = 3;
                        $movimientoCredito2->importemovimiento = $request->enganche;
                        $movimientoCredito2->usuario_id = $usuarioSistema->data->id;
                        $movimientoCredito2->save();
                    }

                    // actualizamos el status del credito del clitente ctl_creditocliente
                    
                    $creditoCliente = new ControlCreditoCliente;
                    $creditoCliente->sucursal_id = $request->sucursal_id;
                    $creditoCliente->cliente_id = $request->cliente_id;
                    $creditoCliente->tipomovimiento_id = 2;
                    $creditoCliente->importeultimomovimiento = $request->total;
                    $creditoCliente->fechaultimacompra = $fechaHoy;
                    $creditoCliente->save();

                    // guardamos en la cartera la venta a credito mov_cartera
                    $movCartera = new Cartera;
                    $movCartera->sucursal_id = $request->sucursal_id;
                    $movCartera->cliente_id = $request->cliente_id;
                    $movCartera->venta_id = $row->id;
                    $movCartera->fechamovimiento = $fechaHoy;
                    $movCartera->fechatienda = $request->fechatienda;
                    $movCartera->importetotal = $request->total;
                    $movCartera->importesobreprecio = $request->importesobreprecio;
                    $movCartera->descuentototal = $request->descuento;
                    $movCartera->pagoinicial = $request->enganche;
                    $movCartera->saldopendiente = $request->saldopendiente;
                    $movCartera->status = 1;
                    $movCartera->save();

                }

                //registramos el ticket generado.


                DB::commit();
                //formamos la respuesta para respuesta
                $response->data = ["venta_id"=>$row->id];
            } catch (\PDOException $e) {
                DB::rollback();
                Log::info('Exception acode: ' . $e->getCode());
                Log::info('Exception message ' . $e->getMessage());
                $response->notificacion->estado = 500;
                $response->notificacion->mensaje = 'Error al intentar guardar algunos datos';
                $response->notificacion->tipo = 'error';
                $response->data = null;
            } catch (\Exception $e) {
                
                DB::rollback();
                Log::info('Exception acode: ' . $e->getCode());
                Log::info('Exception message ' . $e->getMessage());
                // something went wrong
                $response->notificacion->estado = $e->getCode();
                $response->notificacion->mensaje = $e->getMessage();
                $response->notificacion->tipo = 'error';
                $response->data = null;
            }
            
        });
        return $response;

    }

    private function registrarRecargaTA(Peticion $request, $row, $detalles, $pagos, $usuarioSistema){
        $requestID_TA = '';
        $userNameTA = '';
        $dataConfig = '';
        $dataConfigComision = '';
        $comisionRecargaTA = 0;
        $idclientepos = '';

        $response = new \stdclass();
        $response->notificacion = new \stdclass();
        $response->notificacion->estado = 0;
        $response->notificacion->mensaje = 'Exito';
        $response->notificacion->tipo = 'success';
        $response->data = new \stdclass();

        $respuestaRecarga = new \stdclass();
        $configuracionComision = Configuracion::
                        where('sucursal_id','=',$request->sucursal_id)
                        ->where('tipoconfiguracion_id','=',1)
                        ->first();
        $dataConfigComision = json_decode( $configuracionComision->configuracion, true );
        $comisionRecargaTA = $dataConfigComision['comisionRecargaTA'];

        $configuracion = Configuracion::
                        where('sucursal_id','=',$request->sucursal_id)
                        ->where('tipoconfiguracion_id','=',2)
                        ->first();
        $dataConfig = json_decode( $configuracion->configuracion, true );
        $userNameTA = $dataConfig['usr'];
        $idclientepos = $dataConfig['idpos'];
        // Paso 1: obtenemos el requestid para iniciar recarga
        $requestID_TA = $this->getTRequestID($request, $userNameTA, $idclientepos);
        if ( $requestID_TA != ''){
            Log::info("IDcliente:" . $idclientepos . " Iniciando recarga de TA con TRequestID : " . $requestID_TA);
            // Paso 2: Solicitamos la recarga al proveedor 
            $respuestaRecarga =  $this->recargarTAServiciosPOS($request, $requestID_TA, $userNameTA, $idclientepos, $respuestaRecarga);

            if($respuestaRecarga->notificacion->estado == 0){
                Log::info( 'Registrando en la venta ' . json_encode($respuestaRecarga));
                DB::transaction(function() use ($request, $usuarioSistema, $row, $detalles, $pagos, &$response, $respuestaRecarga, $comisionRecargaTA ) {
            
                    try{
                        //grabamos la venta
                        $row->save();
            
                        // obtenemos el detalle de la venta
                        foreach ($detalles as $item) {
                            $detalle = new VentaDetalle;
                            $detalle->venta_id = $row->id;
                            $detalle->codigoarticulo = $item->codigoarticulo;
                            $detalle->tipoproducto = 3;
                            $detalle->cantidad = 1;
                            $detalle->preciounitario = $item->preciounitario;
                            $detalle->descuento = 0;
                            $detalle->iva = 0;
                            $detalle->total = $item->total;
                            $detalle->posicion = 1;
                            $detalle->costo = $item->preciounitario;
                            $detalle->status = 1;
                            $detalle->save();
                            
                            
                            // $resultActualizaTemporal = $this->actualizarTemporalRecarga($params, $telefono, $respuestaRecarga['op_authorization'], $respuestaRecarga['transaction_id']);

                            // guardar la venta de recarga
                            $movimiento = new RecargaTiempoAire;
                            $movimiento->sucursal_id = $request->sucursal_id; 
                            $movimiento->productotiempoaire_id = $item->codigoarticulo;
                            $movimiento->numeroticket = $row->id;
                            $movimiento->cajero_id = $usuarioSistema->data->id;
                            $movimiento->fechatienda = $request->fechatienda;
                            $movimiento->numerotelefono = $item->numerotelefono;
                            $movimiento->opauthorization = $respuestaRecarga->data->op_authorization;
                            $movimiento->transactionid = $respuestaRecarga->data->transaction_id;
                            $movimiento->comision =  $comisionRecargaTA;
                            $movimiento->save();
                        }
            
                        foreach ($pagos as $item) {
                            $pagos = new MovimientoPago;
                            $pagos->referencia_id = $row->id;
                            $pagos->origen_id = $item->origen_id;
                            $pagos->formapago_id = $item->formapago_id;
                            $pagos->importe = $item->importe;
                            $pagos->importeentrega = $item->importeentrega;
                            $pagos->save();
                        }

                        //registramos el ticket generado.
            
                        DB::commit();
                        //formamos la respuesta para respuesta
                        $response->data = ["venta_id"=>$row->id];

                    } catch (\PDOException $e) {
                        DB::rollback();
                        Log::info('Exception acode: ' . $e->getCode());
                        Log::info('Exception message ' . $e->getMessage());
                        $response->notificacion->estado = 200;
                        $response->notificacion->mensaje = 'La recarga SI fue realizada, pero no pudo registrarse en el sistema';
                        $response->notificacion->tipo = 'warning';
                        $response->data = null;
                    } catch (\Exception $e) {
                        
                        DB::rollback();
                        Log::info('Exception acode: ' . 500);
                        Log::info('Exception message ' . $e->getMessage());
                        // something went wrong
                        $response->notificacion->estado = 200;
                        $response->notificacion->mensaje = 'La recarga SI fue realizada, pero no pudo registrarse en el sistema';
                        $response->notificacion->tipo = 'warning';
                        $response->data = null;
                    }

                    
                });

                return $response;
            }else{
                Log::info( 'NO se hizo recarga : ' . json_encode($respuestaRecarga));
                $response->notificacion->estado = $respuestaRecarga->notificacion->estado;
                $response->notificacion->mensaje = $respuestaRecarga->notificacion->mensaje;
                $response->notificacion->tipo = $respuestaRecarga->notificacion->tipo;
                $response->data = $respuestaRecarga->data;
                return $response;
            }
        }else{
            $response->notificacion->estado = 500;
            $response->notificacion->mensaje = 'No fue posible iniciar el proceso de recarga de Tiempo Aire';
            $response->notificacion->tipo = 'warning';
            $response->data = -1;
            return $response;
        }
    }

    public function getTRequestID(Peticion $request, $userNameTA, $idclientepos)
    {
        $url = $this->servicio . "/requestid";
        $query = [
                'username'=>$userNameTA, 
                'idclientepos'=>$idclientepos,
            ];
            // PRUEBAS
            return '06b0b88ed54949b293a0bd3a4dccba55';
        try{
            $client = new Client(['base_uri' => $url,]);
            $clientResponse = $client->request('GET', 'requestid', ['query' => $query]);
            $code = $clientResponse->getStatusCode();
            if($code == 200){
                $clientResponseData = $clientResponse->getBody();
                $remainingBytes = $clientResponseData->getContents();
                $resp = json_decode($remainingBytes);
                return $resp->data;
            }else{
                return '';
            }
        }catch(\Exception $e ){
            Log::info("Exception al consumir serviciospos " . $e);
            return '';
        }   
    }

    public function recargarTAServiciosPOS(Peticion $request, $requestID_TA, $userNameTA, $idclientepos, &$respuestaRecarga)
    {
        $url = $this->servicio;
        $query = ['username'=>$userNameTA, 
                    'idclientepos'=>$idclientepos, 
                    'trequestid'=>$requestID_TA, 
                    'skuCode'=>$request->skucode, 
                    'numerotelefono'=>$request->numerotelefono,
                    'montorecarga'=>$request->monto
                ];

            //PRUEBAS
            $PRUEBAS = new \stdClass();
            $PRUEBAS->rcode = 0;
            $PRUEBAS->op_authorization = 111111;
            $PRUEBAS->transaction_id = 222222;
            $this->respuesta->notificacion->estado = 0;
            $this->respuesta->notificacion->mensaje = 'RECARGA REALIZADO';
            $this->respuesta->notificacion->tipo = 'SUCCESS';
            $this->respuesta->data = $PRUEBAS;
            return $this->respuesta;

        try{

            $client = new Client(['base_uri' => $url,]);
            $clientResponse = $client->request('POST', '', ['form_params' => $query]);
            $code = $clientResponse->getStatusCode();
            $clientResponseData = $clientResponse->getBody();
            $remainingBytes = $clientResponseData->getContents();
            $resp = json_decode($remainingBytes);
            Log::info("code = ...." . $code);
            if($code == 200){
                if($resp->data->rcode == 0 || $resp->data->rcode == 1){
                        // $resultActualizaTemporal = $this->actualizarTemporalRecarga($params, $telefono, $respuestaRecarga['result']['op_authorization'], $respuestaRecarga['result']['transaction_id']);
                        $this->respuesta->notificacion->estado = 0;
                        $this->respuesta->notificacion->mensaje = 'Recarga realizada';
                        $this->respuesta->notificacion->tipo = 'success';
                        $this->respuesta->data = $resp->data;
                        return $this->respuesta;
                }else{
                        Log::error(" El servicio apiserviciospos no pudo completar la recarga de tiempo aire");
                        Log::error(" respuestaRecarga = " . json_encode($resp->data));  
                } 
            }
            $this->respuesta->notificacion->estado = 1;
            $this->respuesta->notificacion->mensaje = $resp->notificacion->mensaje;
            $this->respuesta->notificacion->tipo = $resp->notificacion->tipo;
            $this->respuesta->data = $resp->data;
            return $this->respuesta;
        }catch(\Exception $e ){
            Log::info("Exception service ....");
            $this->respuesta->notificacion->estado = -1;
            $this->respuesta->notificacion->mensaje = 'No fue posible realizar la recarga';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = -1;
            return $this->respuesta;
        }

        
    }

    /**
     * Cortes
     * Se utiliza para obtener todos los cajeros que hicieron ventas en el dia para
     * generar el corte de caja de cada uno
     */
    public function cajerosActivosCobranza(Peticion $request) {

        $params = [];
        $params[] = ['sucursal_id', '=', $request->sucursal_id];
        $params[] = ['tipoconfiguracion_id', '=', 2]; //configuracion de sucursal
        $config = Configuracion::where($params)->first();
        Log::info('config = ' . json_encode($config));
        Log::info('tipo = ' . $config->tipoconfiguracion_id);
        Log::info('tipo = ' . $config->configuracion);
        //Log::info('tipo = ' . $config->configuracion->fechaTienda);
        if(!$config){
            $this->respuesta->notificacion->mensaje = 'Configuración de sucursal inválida';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $config;
            return response()->json($this->respuesta,
                400
            );
        }
        $valorConfiguracion = json_decode($config->configuracion);
        //Log::info('valorConfiguracion = ' .  json_decode($config->configuracion));
        $fechaTienda = $valorConfiguracion->fechaTienda;
        Log::info('fechaTienda = ' .  $fechaTienda);
        $params = [];
        $params[] = ['sucursal_id', '=', $request->sucursal_id];
        $params[] = ['fechatienda', '=', $fechaTienda];

        $data =  DB::table('mov_ventas')
                ->leftJoin('cat_usuarios', 'mov_ventas.cajero_id', '=', 'cat_usuarios.id')
                ->select('mov_ventas.cajero_id', 'cat_usuarios.nombre', 'cat_usuarios.apellidopaterno')
                ->where($params)
                ->groupBy('mov_ventas.cajero_id', 'cat_usuarios.nombre', 'cat_usuarios.apellidopaterno')
                ->get();

        if(!$data){
            $this->respuesta->notificacion->mensaje = 'No se encontro informacion';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                404
            );
        }
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );


      }

    /**
     * Corte de caja cajero: se utiliza en el corte de caja x cajero para obtener los totales de ingresos
     * @param idUsuario req.query.id: el cajero del cual se desea obtener la informacion
     * @param categoria req.query.categoria: el tipo de serie para obtener el corte Todos:0-A:1-B:2
     */
    public function getIngresos(Peticion $request, $fechaTienda) {

        $params = [];
        $params[] = ['sucursal_id', '=', $request->sucursal_id];
        $params[] = ['cajero_id', '=', $request->cajero_id];
        $params[] = ['fechatienda', '=', $fechaTienda];

        return DB::table('mov_ventas')
                ->leftJoin('cat_usuarios', 'mov_ventas.cajero_id', '=', 'cat_usuarios.id')
                ->select('mov_ventas.cajero_id', 'cat_usuarios.nombre', 'cat_usuarios.apellidopaterno', DB::raw('SUM(total) as totalingresos'))
                ->where($params)
                ->first();
    }

    /**
     * Corte caja x cajero: se utiliza para obtener un detalle de los ingresos en caja
     */
    public function detalleIngresos(Peticion $request, $fechaTienda) {
        
        $ingresos = [];
        $params = [];
        $params[] = ['sucursal_id', '=', $request->sucursal_id];
        $params[] = ['cajero_id', '=', $request->cajero_id];
        $params[] = ['fechatienda', '=', $fechaTienda];
        $params[] = ['condicionventa_id', '=', 1];

        //obtener total ingresos ventas a contado
        $totalVentasContado = DB::table('mov_ventas')
                ->select( DB::raw('SUM(total) as total'))
                ->where($params)
                ->first();
        if($totalVentasContado){
            $ingresos[] = ['Ventas a contado' => $totalVentasContado];
        }else{
            $ingresos[] = ['Ventas a contado' => 0];
        }
        
        //obtener total ingresos ventas a credito
        $params = [];
        $params[] = ['sucursal_id', '=', $request->sucursal_id];
        $params[] = ['cajero_id', '=', $request->cajero_id];
        $params[] = ['fechatienda', '=', $fechaTienda];
        $params[] = ['condicionventa_id', '=', 2];
        $totalVentasCredito = DB::table('mov_ventas')
                ->select( DB::raw('SUM(enganche) as total'))
                ->where($params)
                ->first();
        if($totalVentasCredito){
            $ingresos[] = ['Enganche venta crédito' => $totalVentasCredito];
        }else{
            $ingresos[] = ['Enganche venta crédito' => 0];
        }
        //obtener total ingresos ventas tiempo aire
        $params = [];
        $params[] = ['sucursal_id', '=', $request->sucursal_id];
        $params[] = ['cajero_id', '=', $request->cajero_id];
        $params[] = ['fechatienda', '=', $fechaTienda];
        $params[] = ['condicionventa_id', '=', 2];
        $totalVentasTiempoAire = DB::table('mov_ventatiempoaire')
                ->select( DB::raw('SUM(importe + comision) as total'))
                ->where($params)
                ->first();
        if($totalVentasTiempoAire){
            $ingresos[] = ['Ventas de tiempo aire' => $totalVentasTiempoAire];
        }else{
            $ingresos[] = ['Ventas de tiempo aire' => 0];
        }

        return $ingresos;
    }


    /**
     * Sync, envia desde el cliente las ventas almacenadas en local hacia el servidor remoto,
     * se utiliza siempre
     */
    public function sync(Peticion $request)
    {
        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);
        $detalles = array();
        $pagos = array();
        $row = new Venta;

        $params[] = ['_uuid', '=', $request->_uuid];
        $ventaRegistrada =  Venta::where($params)
        ->first();

        //verificamos primero si la venta ya fue replicada al servidor anteriormente, ya no grabar
        if($ventaRegistrada){
            Log::info('Venta ya regsitrada al intentar syncronizar : ' . json_encode($request) );
            $this->respuesta->notificacion->mensaje = 'Venta ya registrada en un proceso anterior';
            $this->respuesta->notificacion->tipo = 'success';
            $this->respuesta->data = $request->_uuid;
            return response()->json($this->respuesta,
                200
            );
        }

        // recuperamos los datos de la venta que llega en la petición
        $this->recuperarDatos($request, $row, $detalles, $pagos);

        if ( $request->tipopago_id == 1){
            // si es venta de producto
            $respuestaVenta = $this->registrarVenta($request, $row, $detalles, $pagos, $usuarioSistema);

            if( $respuestaVenta->notificacion->estado == 0){
                $this->respuesta->notificacion->mensaje = $respuestaVenta->notificacion->mensaje;
                $this->respuesta->notificacion->tipo = $respuestaVenta->notificacion->tipo;
                $this->respuesta->data = $respuestaVenta->data;
                return response()->json($this->respuesta,
                    200
                );
            }else{
                $this->respuesta->notificacion->mensaje = $respuestaVenta->notificacion->mensaje;
                $this->respuesta->notificacion->tipo = $respuestaVenta->notificacion->tipo;
                $this->respuesta->data = $respuestaVenta->data;
                return response()->json($this->respuesta,
                    $respuestaVenta->notificacion->estado
                );
            }


        }

    }

}
