<?php

namespace App\Http\Controllers\Catalogos;

use App\Models\Catalogos\Categoria;
use App\Models\Catalogos\Articulo;
use App\Models\Catalogos\ArticuloPaquete;
use App\Models\Inventario\Inventario;
use App\Models\Inventario\MovimientoArticulo;
use App\Http\Controllers\Controller as Controller;
use Illuminate\Http\Request as Peticion;
use Firebase\JWT\JWT;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use Log;

class ArticuloController extends Controller
{
    private $respuesta;
    public function __construct()
    {
        //
        $this->respuesta = new \stdclass();
        $this->respuesta->notificacion = new \stdclass();
        $this->respuesta->notificacion->mensaje = '';
        $this->respuesta->data = new \stdclass();
    }

    public function show(Peticion $request)
    {
        $per_page = 10;
        $params = [];
        $paramsInv = [];
        $sorting = 'cat_articulos.nombre';
        $order = 'desc';
        if ($request->has('per_page')) {
            $per_page = $request->per_page;
        }
        if ($request->has('order')) {
            $order = $request->order;
        }
        if ($request->has('sorting')) {
            $sorting = $request->sorting;
        }

        if (!is_null($request->getQueryString())) {
            $params = [];
            if ($request->has('id') || $request->has('codigoarticulo')) {
                if($request->has('id')){
                    $params[] = ['cat_articulos.id', '=', $request->id];
                    $paramsInv[] = ['mae_inventarios.articulo_id', '=', $request->id];
                }
                if($request->has('codigoarticulo')){
                    $params[] = ['cat_articulos.codigoarticulo', '=', $request->codigoarticulo];
                    $paramsInv[] = ['mae_inventarios.codigoarticulo', '=', $request->codigoarticulo];
                }
                
                if($request->has('sucursal_id')){
                    
                    $paramsInv[] = ['mae_inventarios.sucursal_id', '=', $request->sucursal_id];
                    /*
                    $data = DB::table('cat_articulos')
                    ->leftJoin('mae_inventarios', 'cat_articulos.id', '=', 'mae_inventarios.articulo_id')
                    ->leftJoin('cat_articulocaracteristicas', 'cat_articulos.id', '=', 'cat_articulocaracteristicas.articulo_id')
                    //->join('cat_articulospaquete', 'cat_articulos.id', '=', 'cat_articulospaquete.articulo_id')
                    ->leftJoin('cat_categorias', 'cat_categorias.id', '=', 'cat_articulos.categoria_id')
                    ->leftJoin('cat_unidadesmedida', 'cat_unidadesmedida.id', '=', 'cat_articulos.unidadmedida_id')
                    //->join('cat_unidadesmedida', 'cat_unidadesmedida.id', '=', 'cat_articulos.unidadmedida_id')
                    ->where($params)
                    ->select('cat_articulos.*', 'mae_inventarios.* as inventario')
                    ->get();
                    */
                    $data =  Articulo::where($params)
                    ->with('departamento')
                    ->with('categoria')
                    ->with('caracteristicas')
                    ->with('unidadmedida')
                    ->with('tipoetiqueta')
                    ->with('inventarios')
                    //->with('articulospaquete')
                    ->first();
                }else{
                    $data =  Articulo::where($params)
                    ->with('departamento')
                    ->with('categoria')
                    ->with('caracteristicas')
                    ->with('unidadmedida')
                    ->with('tipoetiqueta')
                    ->with('inventarios')
                    // ->with('articulospaquete')
                    ->first();
                }

                //poner los articulos del paquete manualmente ya que se permite cantidad
                $data->inventarios()->where($paramsInv);
                if($data->espaquete > 0){
                    $articulosPaquete =  ArticuloPaquete::where('articulo_id','=',$data->espaquete);
                    $arts = [];
                    foreach($articulosPaquete as $art){
                        $a =  Articulo::find($art->articulo_id);
                        $a->cantidad = $art->cantidad;
                        $arts[] = $a;
                    }
                    $data->articulosPaquete = $arts;
                }

                $this->respuesta->data = $data;
                
            }else{
                if ($request->has('nombre')) {
                    $params[] = ['nombre', 'ilike', "%".$request->nombre."%"];
                }
                if(sizeof($params)> 0){
                    $data = Articulo::where($params)
                    ->with('departamento')
                    ->with('categoria')
                    ->with('unidadmedida')
                    ->with('tipoetiqueta')
                    ->orderBy($sorting, $order)
                    ->paginate($per_page);
                    $this->respuesta->data = $data;
                }else{
                    $data = Articulo::with('categoria')
                    ->with('departamento')
                    ->with('unidadmedida')
                    ->with('tipoetiqueta')
                    ->orderBy($sorting, $order)
                    ->paginate($per_page);
                    $this->respuesta->data = $data;
                }

            }

        }else{
            $data = Articulo::with('categoria')
            ->with('unidadmedida')
            ->with('tipoetiqueta')
            ->orderBy($sorting, $order)
            ->paginate($per_page);
            $this->respuesta->data = $data;
        }

        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        // $this->respuesta->data = $data;

        return response()->json($this->respuesta,
            200
        );

    }

    public function find($id)
    {
        $data = Articulo::
        with('categoria')
        ->with('unidadmedida')
        ->with('tipoetiqueta')
        ->findOrFail($id);
        if(!$data){
            $this->respuesta->notificacion->mensaje = 'No se encontro informacion';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                404
            );
        }
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function store(Peticion $request)
    {
        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);

        //verificar si ya existe el registro con el mismo codigo SKU
        $params[] = ['codigoarticulo', '=', $request->codigoarticulo];
        //$params[] = ['status', '=', '1'];
        $rowFound = Articulo::where($params)
                    ->first();

        if($rowFound){
                $data = new \stdClass;
                $data->id = $rowFound->id;
                $this->respuesta->notificacion->estado = 2;
                $this->respuesta->notificacion->mensaje = 'Ya existe un articulo registrado con el mismo SKU';
                $this->respuesta->notificacion->tipo = 'error';
                $this->respuesta->data = $data;
                return response()->json($this->respuesta,
                    400
                );
        }

        //guardamose l articulo
        $row = new Articulo;
        $this->recuperarDatos($request, $row);
        $row->save();
        
        //Guardamos el inventario del articulo
        if($request->has('inventario')){
            $inventario = new Inventario;
            $datosInventario = json_decode(json_encode($request->inventario));
            $inventario->articulo_id = $row->id;
            $inventario->codigoarticulo = $request->codigoarticulo;
            $inventario->preciocosto = $datosInventario->preciocosto ? $datosInventario->preciocosto :0;
            $inventario->preciodeventa = $datosInventario->preciodeventa ? $datosInventario->preciodeventa : 0;
            $inventario->stockminimo = $datosInventario->stockminimo ? $datosInventario->stockminimo : 0;
            $inventario->stockmaximo = $datosInventario->stockmaximo ? $datosInventario->stockmaximo: 0;
            $inventario->existencia = $datosInventario->existencia ? $datosInventario->existencia :0;
            $inventario->iva_id = $datosInventario->iva_id ? $datosInventario->iva_id: 0;
            $inventario->margenutilidad = $datosInventario->margenutilidad ? $datosInventario->margenutilidad :0;
            $inventario->solicitalotesurtido = $datosInventario->solicitalotesurtido ? $datosInventario->solicitalotesurtido : 0;
            $inventario->solicitaadicionalessurtido = $datosInventario->solicitaadicionalessurtido ? $datosInventario->solicitaadicionalessurtido: 0;
            $inventario->sucursal_id = $datosInventario->sucursal_id ? $datosInventario->sucursal_id : 1;
            $inventario->save();

            //registramos kardex de articulo
            $movimiento = new MovimientoArticulo;
            $precioCosto = $datosInventario->preciocosto ? $datosInventario->preciocosto : 0;
            $existenciaNueva = $datosInventario->existencia ? $datosInventario->existencia : 0;
            $movimiento->sucursal_id = $datosInventario->sucursal_id ? $datosInventario->sucursal_id : 1; 
            $movimiento->referencia_id = $row->id;
            $movimiento->tipomovimiento_id = 7; //alta articulo, 
            $movimiento->codigoarticulo = $request->codigoarticulo;
            $movimiento->cantidad = $datosInventario->existencia ? $datosInventario->existencia: 0;
            $movimiento->precioventa = $datosInventario->preciodeventa ? $datosInventario->preciodeventa: 0;
            $movimiento->preciocosto = $datosInventario->preciocosto ? $datosInventario->preciocosto: 0;
            $movimiento->usuario_id = $usuarioSistema->data->id;
            $movimiento->descuento =  0;
            $movimiento->importe = ($precioCosto * $existenciaNueva);
            $movimiento->save();
        }
        //Guardamos los ariticulos del paquete cuando es paquete
        if($request->has('articulospaquete')){
            $articulosPaquete = json_decode(json_encode($request->articulospaquete));
            foreach ($articulosPaquete as $item) {
                $paquete = new ArticuloPaquete;
                $paquete->articulo_id = $row->id;
                $paquete->codigoarticulo = $item->codigoarticulo;
                $paquete->cantidad = $item->cantidad;
                $paquete->save();
            }
        }

        
        
        //formamos la respuesta para respuesta
        $data = new \stdClass;
        $data->id = $row->id;
        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function update($id, Peticion $request)
    {
        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);
        
        //verificar si ya existe el registro con el mismo codigo SKU
        $params[] = ['codigoarticulo', '=', $request->codigoarticulo];
        //$params[] = ['status', '=', '1'];
        $row = Articulo::where($params)
                    ->first();

        if(!$row){
                $data = new \stdClass;
                $data->id = $row->id;
                $this->respuesta->notificacion->estado = 2;
                $this->respuesta->notificacion->mensaje = 'Este articulo no existe';
                $this->respuesta->notificacion->tipo = 'error';
                $this->respuesta->data = $data;
                return response()->json($this->respuesta,
                    404
                );
        }

        $this->recuperarDatos($request, $row);
        $row->save();
        
        //Guardamos el inventario del articulo
        if($request->has('inventario')){
            $datosInventario = json_decode(json_encode($request->inventario));

            $params = [];
            $params[] = ['codigoarticulo', '=', $request->codigoarticulo];
            $params[] = ['sucursal_id', '=', $datosInventario->sucursal_id];
            $inventario = Inventario::where($params)
                        ->first();
            if(!$inventario){
                $inventario = new Inventario;
            }
            Log::info(' inventario = ' . json_encode($inventario));

            
            $inventario->articulo_id = $id;
            $inventario->codigoarticulo = $request->codigoarticulo;
            $inventario->preciocosto = $datosInventario->preciocosto ? $datosInventario->preciocosto: 0;
            $inventario->preciodeventa = $datosInventario->preciodeventa ? $datosInventario->preciodeventa:0;
            $inventario->stockminimo = $datosInventario->stockminimo ? $datosInventario->stockminimo: 0;
            $inventario->stockmaximo = $datosInventario->stockmaximo ? $datosInventario->stockmaximo: 0;
            $inventario->existencia = $datosInventario->existencia ? $datosInventario->existencia: 0;
            $inventario->iva_id = $datosInventario->iva_id ? $datosInventario->iva_id: 0;
            $inventario->margenutilidad = $datosInventario->margenutilidad ? $datosInventario->margenutilidad: 0;
            $inventario->solicitalotesurtido = $datosInventario->solicitalotesurtido ? $datosInventario->solicitalotesurtido: 0;
            $inventario->solicitaadicionalessurtido = $datosInventario->solicitaadicionalessurtido ? $datosInventario->solicitaadicionalessurtido:0;
            $inventario->sucursal_id = $datosInventario->sucursal_id ? $datosInventario->sucursal_id: 1;
            $inventario->save();

            //registramos kardex de articulo
            $movimiento = new MovimientoArticulo;
            $precioCosto = $datosInventario->preciocosto ? $datosInventario->preciocosto : 0;
            $existenciaNueva = $datosInventario->existencia ? $datosInventario->existencia : 0;
            $movimiento->sucursal_id = $datosInventario->sucursal_id; 
            $movimiento->referencia_id = $row->id;
            $movimiento->tipomovimiento_id = 7; //alta articulo, 
            $movimiento->codigoarticulo = $request->codigoarticulo;
            $movimiento->cantidad = $datosInventario->existencia;
            $movimiento->precioventa = $datosInventario->preciodeventa;
            $movimiento->preciocosto = $datosInventario->preciocosto;
            $movimiento->usuario_id = $usuarioSistema->data->id;
            $movimiento->descuento =  0;
            $movimiento->importe = ($precioCosto * $existenciaNueva);
            $movimiento->save();
        }
        //Guardamos los ariticulos del paquete cuando es paquete
        if($request->has('articulosPaquete')){
            $params = [];
            $params[] = ['articulo_id', '=', $id];
            $rowsPaquete = ArticuloPaquete::where($params)
                        ->first();
            if($rowsPaquete){
                $rowsPaquete = $rowsPaquete->delete();
            }

            $articulosPaquete = json_decode(json_encode($request->articulosPaquete));
            foreach ($articulosPaquete as $item) {
                $paquete = new ArticuloPaquete;
                $paquete->articulo_id = $id;
                $paquete->codigoarticulo = $item->codigoarticulo;
                $paquete->cantidad = $item->cantidad;
                $paquete->save();
            }
        }

        
        
        //formamos la respuesta para respuesta
        $data = new \stdClass;
        $data->id = $row->id;

        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );


    }

    public function delete($id)
    {
        $cliente = Articulo::findOrFail($id);
        $result = $cliente->delete();

        $data = new \stdClass;
        $data->id = $newRow->id;
        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $result;
        return response()->json($this->respuesta,
            200
        );
    }

    private function recuperarDatos(Peticion $request, &$registroGuardar){

        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);
        //usuario que captura el registro
        $registroGuardar->usuario_id = $usuarioSistema->data->id;

        
        if ($request->has('codigoarticulo')) {
            $registroGuardar->codigoarticulo = $request->codigoarticulo;
        }
        if ($request->has('nombre')) {
            $registroGuardar->nombre = $request->nombre;
        }
        if ($request->has('tipocodigo_id')) {
            $registroGuardar->tipocodigo_id = $request->tipocodigo_id;
        }
        if ($request->has('departamento_id')) {
            $registroGuardar->departamento_id = $request->departamento_id;
        }
        if ($request->has('categoria_id')) {
            $registroGuardar->categoria_id = $request->categoria_id;
        }
        if ($request->has('marca_id')) {
            $registroGuardar->marca_id = $request->marca_id;
        }
        if ($request->has('modelo')) {
            $registroGuardar->modelo = $request->modelo;
        }
        if ($request->has('color')) {
            $registroGuardar->color = $request->color;
        }
        if ($request->has('contenido')) {
            $registroGuardar->contenido = $request->contenido;
        }
        if ($request->has('unidadmedida_id')) {
            $registroGuardar->unidadmedida_id = $request->unidadmedida_id;
        }
        if ($request->has('tipoetiqueta_id')) {
            $registroGuardar->tipoetiqueta_id = $request->tipoetiqueta_id;
        }
        if ($request->has('tipocodigo_id')) {
            $registroGuardar->tipocodigo_id = $request->tipocodigo_id;
        }
        if ($request->has('status')) {
            $registroGuardar->status = $request->status;
        }
        if ($request->has('tags')) {
            $tag = implode(' ', $request->tags);
            $tag = '' .$tag;
            $registroGuardar->tags =  $tag;
        }
        
        if ($request->has('articulosPaquete')) {
            $datospaquete = json_decode(json_encode($request->articulosPaquete));
            if(sizeof($datospaquete) > 0){
                $registroGuardar->espaquete = 1;
            }
        }
        /*
        if ($request->has('inventario')) {
            $registroGuardar->inventario = json_decode(json_encode($request->inventario));
        }
        */

    }

    public function getItemVenta(Peticion $request)
    {

        if (!is_null($request->getQueryString())) {
            
            $data = Articulo::where('codigoarticulo', $request->codigoarticulo)
                ->with('unidadmedida')
                ->select('codigoarticulo', 'unidadmedida_id', 'nombre', 'modelo', 'contenido', 'id')
                ->first();
            $this->respuesta->notificacion->mensaje = 'Exito';
            $this->respuesta->notificacion->tipo = 'success';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                200
            );

        }else{
            $this->respuesta->notificacion->mensaje = 'Debe proporcionar un codigo de articulo';
            $this->respuesta->notificacion->tipo = 'error';
            $this->respuesta->data = null;
            return response()->json($this->respuesta,
                400
            );
        }

    }

    /**
     * Obtiene catalogo paginado para descargar para guardar local en modo offline
     */
    public function getForOffLine(Peticion $request)
    {
        $per_page = 10;
        $params = [];
        $paramsInv = [];
        $sorting = 'cat_articulos.id';
        $order = 'asc';
        if ($request->has('per_page')) {
            $per_page = $request->per_page;
        }
                
        if($request->has('sucursal_id')){
                    
            $paramsInv[] = ['mae_inventarios.sucursal_id', '=', $request->sucursal_id];
            
            $data = DB::table('cat_articulos')
            ->join('mae_inventarios', 'cat_articulos.id', '=', 'mae_inventarios.articulo_id')
            ->leftJoin('cat_articulocaracteristicas', 'cat_articulos.id', '=', 'cat_articulocaracteristicas.articulo_id')
            //->join('cat_articulospaquete', 'cat_articulos.id', '=', 'cat_articulospaquete.articulo_id')
            ->leftJoin('cat_categorias', 'cat_categorias.id', '=', 'cat_articulos.categoria_id')
            ->leftJoin('cat_unidadesmedida', 'cat_unidadesmedida.id', '=', 'cat_articulos.unidadmedida_id')
            ->where($params)
            ->select('cat_articulos.*', 'mae_inventarios.* as inventario')
            ->orderBy($sorting, $order)
            ->paginate($per_page);
            $this->respuesta->notificacion->mensaje = 'Exito';
            $this->respuesta->notificacion->tipo = 'success';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                200
            );
            
        }

                

        
        $this->respuesta->notificacion->mensaje = 'La sucursal no es valida';
        $this->respuesta->notificacion->tipo = 'error';
        $this->respuesta->data = null;
        return response()->json($this->respuesta,
            200
        );

    }

}
