<?php

namespace App\Http\Controllers\Catalogos;

use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Http\Request as Peticion;
use App\Http\Controllers\Controller as Controller;
use Illuminate\Support\Facades\DB;
use App\Models\Catalogos\Caja;

class CajaController extends Controller
{
    private $respuesta;
    public function __construct()
    {
        $this->respuesta = new \stdclass();
        $this->respuesta->notificacion = new \stdclass();
        $this->respuesta->notificacion->estado = 0;
        $this->respuesta->notificacion->mensaje = '';
        $this->respuesta->data = new \stdclass();
    }

    public function show(Peticion $request)
    {  
        $per_page = 10;
        $params = [];
        $sorting = 'id';
        $order = 'asc'; // desc
        return Caja::orderBy($sorting, $order)
                ->paginate($per_page);
    }

    public function find($id)
    {
        $data = Usuario::with('perfil')->find($id);
        if(!$data){
            $this->respuesta->notificacion->estado = 1;
            $this->respuesta->notificacion->mensaje = 'Registro no encontrado';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                404
            );
        }

        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );

    }

    public function store(Peticion $request)
    {
        $usuario = new Usuario;
        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);

        $usuario->nombre = $request->nombre;
        $usuario->apellidopaterno = $request->apellidopaterno;
        if ($request->has('apellidomaterno')) {
            $usuario->apellidomaterno = $request->apellidomaterno;
        }
        if ($request->has('status')) {
            $usuario->status = $request->status;
        }
        $usuario->perfil_id = $request->perfil_id;
        $usuario->pwd = $request->pwd;
        
        $usuario->usuario_id = $usuarioSistema->data->id;
        $usuario->save();

        $usuario->save();

        $this->respuesta->data = ["id"=>$usuario->id];
        $this->respuesta->notificacion->mensaje = 'Registro almacenado con exito';
        $this->respuesta->notificacion->tipo = 'success';
        return response()->json($this->respuesta,
            200
        );
    }

    public function update($id, Peticion $request)
    {
        
        
            $usuario = Usuario::find($id);

            $token = $request->header('Authorization');
            $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);

            if ($request->has('perfil_id')) {
                $usuario->perfil_id = $request->perfil_id;
            }
            if ($request->has('nombre')) {
                $usuario->nombre = $request->nombre;
            }
            if ($request->has('apellidopaterno')) {
                $usuario->apellidopaterno = $request->apellidopaterno;
            }
            if ($request->has('apellidomaterno')) {
                $usuario->apellidomaterno = $request->apellidomaterno;
            }
            if ($request->has('status')) {
                $usuario->status = $request->status;
            }
            //usuario que realiza la modificacion de datos
            $usuario->usuario_id = $usuarioSistema->data->id;

            if ($request->has('pwd')) {
                $usuario->pwd = $request->pwd;
                /*
                if ($usuario->pwd === $request->pwd) {
                    $usuario->pwd = $request->has('nuevopwd') ? $request->has('nuevopwd') : $request->pwd;
                } else {
                    return response(['mensaje' =>'Password no coincide.'], 403);
                }
                */
            }

            $usuario->save();


        $this->respuesta->data = ["id"=>$usuario->id];
        $this->respuesta->notificacion->mensaje = 'Registro actualizado con exito';
        $this->respuesta->notificacion->tipo = 'success';
        return response()->json($this->respuesta,
            200
        );

    }

    public function delete($id)
    {
        $usuario = Usuario::find($id);
        if(!$usuario){
            $data = new \stdClass;
            $this->respuesta->notificacion->estado = 1;
            $this->respuesta->notificacion->mensaje = 'Registro no encontrado';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                404
            );
        }
        $data = [
            "result" => $usuario->delete(),
        ];
        $this->respuesta->notificacion->mensaje= 'Registro eliminado correctamente';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function listaKeyValue(Peticion $request)
    {
        $secciones = array();
        $sorting = 'nombre';
        $order = 'asc';
        $params = [];
        $perfiles = array();
        array_push($perfiles, '1'); //administrador
        array_push($perfiles, '6'); //supervisor extrajudicial
        array_push($perfiles, '8'); //supervisor judicial
        if (!is_null($request->getQueryString())) {
            
            if ($request->has('perfil_id')) {
                array_push($perfiles, $request->input('perfil_id'));
            }
            if ($request->has('status')) {
                $params[] = ['status', '=', $request->input('status')];
            }
            

            $resultSet = Usuario::
                whereIn("perfil_id", $perfiles)
                ->where($params)
                ->select('id', 'nombre', 'apellidopaterno', 'perfil_id')
                ->orderBy($sorting, $order)
                ->get();

                return $resultSet;

        } 
        $resultSet = Usuario::
                //where($params)
                select('id', 'nombre', 'apellidopaterno' )
                ->where("status","1")
                ->orderBy($sorting, $order)
                ->get();
        return $resultSet;

    }

    public function login(Peticion $request)
    {
        if ($request->has('id') && $request->has('pwd')) {
            $usuario = Usuario::findOrFail($request->id);
            if ($request->pwd === $usuario->pwd) {
                $token = array(
                    "alg" => "HS256",
                    "typ" => "JWT",
                    "iss" => "http://snowcloud.com",
                    "aud" => "http://snowcloud.com",
                    "iat" => 1356999524,
                    "nbf" => 1357000000,
                    "data" => [
                        "nombre" => $usuario->nombre,
                        "apellidoPaterno" => $usuario->apellidopaterno,
                        "apellidoMaterno" => $usuario->apellidoMaterno,
                        "perfil" => $usuario->perfil_id,
                        "id" => $request->id

                    ]
                );
                $jwt = JWT::encode($token, getenv('APP_KEY'));

                $data = new \stdClass;
                $data = ["token" => $jwt, 
                "usuario_id"=>$request->id, 
                "perfil_id"=>$usuario->perfil_id,
                "nombre"=>$usuario->nombre,
                "apellidoPaterno"=>$usuario->apellidoPaterno];

                //$this->respuesta->notificacion->mensaje = 'Gestion registrada correctamente';
                //$this->respuesta->notificacion->tipo = 'success';
                $this->respuesta->data = $data;
                return response()->json($this->respuesta,
                    200
                );
            } else {
                $data = new \stdClass;
                    $this->respuesta->estado = 1;
                    $this->respuesta->notificacion->mensaje = 'Contraseña inválida';
                    $this->respuesta->notificacion->tipo = 'warning';
                    $this->respuesta->data = $data;
                    return response()->json($this->respuesta,
                        401
                    );
            }
        } else {
            $this->respuesta->estado = 1;
                    $this->respuesta->notificacion->mensaje = 'Credenciales inválidas';
                    $this->respuesta->notificacion->tipo = 'warning';
                    $this->respuesta->data = $data;
                    return response()->json($this->respuesta,
                        401
                    );
        }
    }
}
