<?php

namespace App\Http\Controllers\OperacionesCaja;

use App\Models\OperacionesCaja\CorteCaja; 
use App\Models\Configuracion\Configuracion; 
use App\Http\Controllers\Controller as Controller;
use Illuminate\Http\Request as Peticion;
use Firebase\JWT\JWT;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use Log;

class CorteCajaController extends Controller
{
    private $respuesta;
    public function __construct()
    {
        //
        $this->respuesta = new \stdclass();
        $this->respuesta->notificacion = new \stdclass();
        $this->respuesta->notificacion->mensaje = '';
        $this->respuesta->data = new \stdclass();
    }

    public function show(Peticion $request)
    {
        $per_page = 10;
        $params = [];
        $sorting = 'nombre';
        $order = 'desc';
        if ($request->has('per_page')) {
            $per_page = $request->per_page;
        }
        if ($request->has('order')) {
            $order = $request->order;
        }
        if ($request->has('sorting')) {
            $sorting = $request->sorting;
        }

        if (!is_null($request->getQueryString())) {
            $params = [];
            if ($request->has('id')) {
                $params[] = ['id', '=', $request->id];
                $data =  CorteCaja::where($params)
                ->first();
                $this->respuesta->data = $data;
                
            }else{
                if ($request->has('nombre')) {
                    $params[] = ['nombre', 'ilike', "%".$request->nombre."%"];
                }
                $data = CorteCaja::where($params)
                ->orderBy($sorting, $order)
                ->paginate($per_page);
                $this->respuesta->data = $data;
            }

        }else{
            $data = CorteCaja::
            orderBy($sorting, $order)
            ->paginate($per_page);
            $this->respuesta->data = $data;
        }

        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        return response()->json($this->respuesta,
            200
        );

    }

    public function find($id)
    {
        $data = CorteCaja::find($id);
        if(!$data){
            $this->respuesta->notificacion->mensaje = 'No se encontro informacion';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                404
            );
        }
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function store(Peticion $request)
    {
        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);

        //guardamose l articulo
        $row = new CorteCaja;
        $this->recuperarDatos($request, $row);
        $row->save();
        
        //formamos la respuesta para respuesta
        $data = new \stdClass;
        $data->id = $row->id;
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function update($id, Peticion $request)
    {

        //verificar si ya existe el registro
        $row = CorteCaja::find($id);

        $this->recuperarDatos($request, $row);

        //verificar si cambio la etapa
        $row->save();


        //formamos la respuesta para respuesta
        $data = new \stdClass;
        $data->id = $newRow->id;
        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );


    }

    public function delete($id)
    {
        $row = CorteCaja::find($id);
        $result = $row->delete();

        $data = new \stdClass;
        $data->id = $row->id;
        $this->respuesta->notificacion->mensaje = 'Registro eliminado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $result;
        return response()->json($this->respuesta,
            200
        );
    }

    private function recuperarDatos(Peticion $request, &$registroGuardar){

        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);
        //usuario que captura el registro
        $registroGuardar->usuario_id = $usuarioSistema->data->id;
        if ($request->has('cajero_id')) {
            $registroGuardar->cajero_id = $request->cajero_id;
        }
        if ($request->has('totalingresos')) {
            $registroGuardar->totalingresos = $request->totalingresos;
        }
        if ($request->has('totalegresos')) {
            $registroGuardar->totalegresos = $request->totalegresos;
        }
        if ($request->has('totalentregado')) {
            $registroGuardar->totalentregado = $request->totalentregado;
        }
        if ($request->has('diferencia')) {
            $registroGuardar->diferencia = $request->diferencia;
        }
        if ($request->has('fechatienda')) {
            $registroGuardar->fechatienda = $request->fechatienda;
        }
    }

    public function listaKeyValue(Peticion $request)
    {
        $sorting = 'fechatienda';
        $order = 'desc';
        $params = [];
        $resultSet = CorteCaja::
                select('id', 'fechatienda' )
                ->orderBy($sorting, $order)
                ->get();
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $resultSet;
        return response()->json($this->respuesta,
            200
        );

    }

    /**
     * Cortes de caja por cajero: Se invoca al seleccionar un cajero para verificar si el
     * cajero ya tiene registrado un corte en el dia
     * @param req GET id: es el id del cajero el cual se va verificar corte
     * @param res status: 400 para indicar que ya existe un corte, status: 200 noexiste el corte
    */
    public function verifica(Peticion $request)
    {

        $params = [];

        $params[] = ['sucursal_id', '=', $request->sucursal_id];
        $params[] = ['tipoconfiguracion_id', '=', 2]; //configuracion de sucursal
        $config = Configuracion::where($params)->first();
        if(!$config){
            $this->respuesta->notificacion->mensaje = 'Configuración de sucursal inválida';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                400
            );
        }
        $valorConfiguracion = json_decode($config->configuracion);
        $fechaTienda = $valorConfiguracion->fechaTienda;

        $data = $this->verificaCorte($request, $fechaTienda);
        if($data){
            $this->respuesta->notificacion->mensaje = 'YA EXISTE UN CORTE DE CAJA EL DIA DE HOY PARA EL CAJERO SELECCIONADO';
            $this->respuesta->notificacion->tipo = 'error';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                400
            );
        }
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );

    }

    public function verificaCorte(Peticion $request, $fechaTienda)
    {

        $fechaInicial = '';
        $fechaFinal = '';
        
        $fechaInicial = $fechaTienda . ' 00:00:00';
        $fechaFinal = $fechaTienda . ' 23:59:59';

        $d =  CorteCaja::where('cajero_id','=',$request->cajero_id)
            ->where('fechatienda', '=', $fechaTienda)
            ->first();
            
        //Log::info(' d = ', json_encode($d));
        return $d;
    }

    /**
    * Cortes de caja por cajero: se invica despues de verifica y solo en caso de que verifica
    * retorne un 200, se utiliza para obtener el detalle del corte a mostrar en pantalla
    * @param req GET id: del cajero que se desea hacer corte
    * @param res JSON data: con el resumen de totales de ingresos, egresos y detalles
     */
    public function obtenerInfoCorte(Peticion $request)
    {
        $params = [];
        $ventaController = new VentaController;
        $egresosController = new EgresoController;
        $retirosController = new EgresoController;

        $params[] = ['sucursal_id', '=', $request->sucursal_id];
        $params[] = ['tipoconfiguracion_id', '=', 2]; //configuracion de sucursal
        $config = Configuracion::where($params)->first();
        if(!$config){
            $this->respuesta->notificacion->mensaje = 'Configuración de sucursal inválida';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $config;
            return response()->json($this->respuesta,
                400
            );
        }
        $valorConfiguracion = json_decode($config->configuracion);
        $fechaTienda = $valorConfiguracion->fechaTienda;

        $totales = $ventaController.getIngresos($request, $fechaTienda);
        $totalEgresos = $egresosController.getEgresos($request, $fechaTienda);
        $totalRetiros = $retirosController.getRetiros($request, $fechaTienda);
        $detalleIngresos = $ventaController.detalleIngresos($request, $fechaTienda);
        $detalleEgresos = $egresosController.detalleEgresos($request, $fechaTienda);
        
        $totalEgresosGlobal = ($totalEgresos ? $totalEgresos : 0 ) + ($totalRetiros ? $totalRetiros : 0);

        $salidasCaja = array_merge($totalRetiros, $detalleEgresos);

        $data = new \stdClass();
        $data->usuario_id = $totales->cajero_id;
        $data->nombre = $totales->nombre;
        $data->apellidopaterno = $totales->apellidopaterno;
        $data->totalingresos = $totales->totalingresos ? $totales->totalingresos : 0;
        $data->totalegresos = $totalEgresosGlobal;
        $data->diferencia = $data->totalingresos - $data->totalegresos;
        $data->ingresos = $detalleIngresos;
        $data->egresos = $salidasCaja;

        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );

    }

}
