<?php

namespace App\Http\Controllers\OperacionesCaja;

use App\Models\OperacionesCaja\Egreso; 
use App\Http\Controllers\Controller as Controller;
use Illuminate\Http\Request as Peticion;
use Firebase\JWT\JWT;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;

class EgresoController extends Controller
{
    private $respuesta;
    public function __construct()
    {
        //
        $this->respuesta = new \stdclass();
        $this->respuesta->notificacion = new \stdclass();
        $this->respuesta->notificacion->mensaje = '';
        $this->respuesta->data = new \stdclass();
    }

    public function show(Peticion $request)
    {
        $per_page = 10;
        $params = [];
        $sorting = 'nombre';
        $order = 'desc';
        if ($request->has('per_page')) {
            $per_page = $request->per_page;
        }
        if ($request->has('order')) {
            $order = $request->order;
        }
        if ($request->has('sorting')) {
            $sorting = $request->sorting;
        }

        if (!is_null($request->getQueryString())) {
            $params = [];
            if ($request->has('id')) {
                $params[] = ['id', '=', $request->id];
                $data =  Egreso::where($params)
                ->first();
                $this->respuesta->data = $data;
                
            }else{
                if ($request->has('nombre')) {
                    $params[] = ['nombre', 'ilike', "%".$request->nombre."%"];
                }
                $data = Egreso::where($params)
                ->orderBy($sorting, $order)
                ->paginate($per_page);
                $this->respuesta->data = $data;
            }

        }else{
            $data = Egreso::
            orderBy($sorting, $order)
            ->paginate($per_page);
            $this->respuesta->data = $data;
        }

        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        return response()->json($this->respuesta,
            200
        );

    }

    public function find($id)
    {
        $data = Egreso::find($id);
        if(!$data){
            $this->respuesta->notificacion->mensaje = 'No se encontro informacion';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                404
            );
        }
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function store(Peticion $request)
    {
        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);

        //guardamose l articulo
        $row = new Egreso;
        $this->recuperarDatos($request, $row);
        $row->save();
        
        //formamos la respuesta para respuesta
        $data = new \stdClass;
        $data->id = $row->id;
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function update($id, Peticion $request)
    {

        //verificar si ya existe el registro
        $row = Egreso::find($id);

        $this->recuperarDatos($request, $row);

        //verificar si cambio la etapa
        $row->save();


        //formamos la respuesta para respuesta
        $data = new \stdClass;
        $data->id = $newRow->id;
        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );


    }

    public function delete($id)
    {
        $row = Egreso::find($id);
        $result = $row->delete();

        $data = new \stdClass;
        $data->id = $row->id;
        $this->respuesta->notificacion->mensaje = 'Registro eliminado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $result;
        return response()->json($this->respuesta,
            200
        );
    }

    private function recuperarDatos(Peticion $request, &$registroGuardar){

        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);
        //usuario que captura el registro
        $registroGuardar->usuario_id = $usuarioSistema->data->id;
        if ($request->has('cajero_id')) {
            $registroGuardar->cajero_id = $request->cajero_id;
        }
        if ($request->has('totalingresos')) {
            $registroGuardar->totalingresos = $request->totalingresos;
        }
        if ($request->has('totalegresos')) {
            $registroGuardar->totalegresos = $request->totalegresos;
        }
        if ($request->has('totalentregado')) {
            $registroGuardar->totalentregado = $request->totalentregado;
        }
        if ($request->has('diferencia')) {
            $registroGuardar->diferencia = $request->diferencia;
        }
        if ($request->has('fechatienda')) {
            $registroGuardar->fechatienda = $request->fechatienda;
        }
    }

    public function listaKeyValue(Peticion $request)
    {
        $sorting = 'fechatienda';
        $order = 'desc';
        $params = [];
        $resultSet = Egreso::
                select('id', 'fechatienda' )
                ->orderBy($sorting, $order)
                ->get();
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $resultSet;
        return response()->json($this->respuesta,
            200
        );

    }

      /**
     * Corte caja x cajero: se utiliza para obtener los totales de egresos
     * @param idUsuario req.query.id: es el id del cajero delcual se obtendra la información
     */
    public function getEgresos(Peticion $request, $fechaTienda) {
        
        $fechaInicial = $fechaTienda . ' 00:00:00';
        $fechaFinal = $fechaTienda . ' 23:59:59';

        return Egreso::where('cajero_id','=',$request->cajero_id)
            ->whereBetween('created_at', [$fechaInicial, $fechaFinal])
            ->select(DB::raw('SUM(mov_egresos.importe) as totalegresos'))
            ->groupBy('mov_egresos.cajero_id')
            ->first();
    }

    /**
     * Corte caja x cajero: se utiliza para obtener un detalle de los egresos en caja
     */
    public function detalleEgresos(Peticion $request, $fechaTienda) {
        
        $egresos = [];
        $params = [];
        $params[] = ['sucursal_id', '=', $request->sucursal_id];
        $params[] = ['cajero_id', '=', $request->cajero_id];
        $params[] = ['fechatienda', '=', $fechaTienda];

        //obtener total egresos caja
        $egresosCaja = DB::table('mov_egresos')
                ->join('cat_tipoegreso', 'mov_egresos.tipoegreso_id', 'cat_tipoegreso.id' )
                ->select( 'cat_tipoegreso.descripcion', 'mov_egresos.comentarios', DB::raw('SUM(importe) as total'))
                ->where($params)
                ->get();
        if($egresosCaja){
            foreach ($egresosCaja as $item) {
                $egresos[] = [$item->descripcion . ' ' . $item->comentarios => $item->total];
            }
        }
        

        return $egresos;
    }


}
