<?php

namespace App\Http\Controllers\OperacionesCaja;

use App\Models\OperacionesCaja\Retiro; 
use App\Http\Controllers\Controller as Controller;
use Illuminate\Http\Request as Peticion;
use Firebase\JWT\JWT;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;

class RetiroController extends Controller
{
    private $respuesta;
    public function __construct()
    {
        //
        $this->respuesta = new \stdclass();
        $this->respuesta->notificacion = new \stdclass();
        $this->respuesta->notificacion->mensaje = '';
        $this->respuesta->data = new \stdclass();
    }

    public function show(Peticion $request)
    {
        $per_page = 10;
        $params = [];
        $sorting = 'nombre';
        $order = 'desc';
        if ($request->has('per_page')) {
            $per_page = $request->per_page;
        }
        if ($request->has('order')) {
            $order = $request->order;
        }
        if ($request->has('sorting')) {
            $sorting = $request->sorting;
        }

        if (!is_null($request->getQueryString())) {
            $params = [];
            if ($request->has('id')) {
                $params[] = ['id', '=', $request->id];
                $data =  Retiro::where($params)
                ->first();
                $this->respuesta->data = $data;
                
            }else{
                if ($request->has('nombre')) {
                    $params[] = ['nombre', 'ilike', "%".$request->nombre."%"];
                }
                $data = Retiro::where($params)
                ->orderBy($sorting, $order)
                ->paginate($per_page);
                $this->respuesta->data = $data;
            }

        }else{
            $data = Retiro::
            orderBy($sorting, $order)
            ->paginate($per_page);
            $this->respuesta->data = $data;
        }

        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        return response()->json($this->respuesta,
            200
        );

    }

    public function find($id)
    {
        $data = Retiro::find($id);
        if(!$data){
            $this->respuesta->notificacion->mensaje = 'No se encontro informacion';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                404
            );
        }
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function store(Peticion $request)
    {
        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);

        //guardamos
        $row = new Egreso;
        $this->recuperarDatos($request, $row);
        $row->save();
        
        //formamos la respuesta para respuesta
        $data = new \stdClass;
        $data->id = $row->id;
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function update($id, Peticion $request)
    {

        //verificar si ya existe el registro
        $row = Retiro::find($id);

        $this->recuperarDatos($request, $row);

        //verificar si cambio la etapa
        $row->save();


        //formamos la respuesta para respuesta
        $data = new \stdClass;
        $data->id = $newRow->id;
        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );


    }

    public function delete($id)
    {
        $row = Retiro::find($id);
        $result = $row->delete();

        $data = new \stdClass;
        $data->id = $row->id;
        $this->respuesta->notificacion->mensaje = 'Registro eliminado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $result;
        return response()->json($this->respuesta,
            200
        );
    }

    private function recuperarDatos(Peticion $request, &$registroGuardar){

        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);
        //usuario que captura el registro
        //$registroGuardar->usuario_id = $usuarioSistema->data->id;
        if ($request->has('cajero_id')) {
            $registroGuardar->cajero_id = $request->cajero_id;
        }
        if ($request->has('importe')) {
            $registroGuardar->importe = $request->importe;
        }
        if ($request->has('formapago_id')) {
            $registroGuardar->formapago_id = $request->formapago_id;
        }
        if ($request->has('sucursal_id')) {
            $registroGuardar->sucursal_id = $request->sucursal_id;
        }
    }

      /**
     * Corte caja x cajero: se utiliza para obtener los totales de egresos
     * @param idUsuario req.query.id: es el id del cajero delcual se obtendra la información
     */
    public function getRetiros(Peticion $request, $fechaTienda) {

        $total =  Retiro::where('cajero_id','=',$request->cajero_id)
            ->where('fechatienda', '=', $fechaTienda)
            ->where('sucursal_id', '=', $request->sucursal_id)
            ->select(DB::raw('SUM(importe) as totalretiros'))
            ->first();
        $retiros[] = ['Retiros parciales' => $total];
    }

}
