<?php

namespace App\Http\Controllers\Seguridad;

use App\Models\Seguridad\Rol;
use App\Models\Seguridad\ACL;
use App\Http\Controllers\Controller as Controller;
use Illuminate\Http\Request as Peticion;
use Firebase\JWT\JWT;

class ACLController extends Controller
{
    private $respuesta;
    public function __construct()
    {
        $this->respuesta = new \stdclass();
        $this->respuesta->notificacion = new \stdclass();
        $this->respuesta->notificacion->estado = 0;
        $this->respuesta->notificacion->mensaje = '';
        $this->respuesta->data = new \stdclass();
    }

    public function show(Peticion $request)
    {
        if ($request->has('perfil_id')) {
            $params[] = ['rol_id', '=', $request->input('perfil_id')];
        } 
        $acl = ACL::where($params)
        ->first();
        if (!$acl) {
            return response(
                ['mensaje' => 'No hay una lista de acceso definida para este perfil'],
                400
            );
        }
        $acl->acl = json_decode($acl->acl);
        //return $acl;
        $this->respuesta->data = $acl;
        $this->respuesta->notificacion->mensaje = 'EXITO';
        $this->respuesta->notificacion->tipo = 'success';
        return response()->json($this->respuesta,
            200
        );
    }

    public function find($perfil_id)
    {
        $acl = ACL::where(
            [
                ['rol_id', '=', $perfil_id]
            ]
        )->first();
        if (!$acl) {
            return response(
                ['mensaje' => 'No hay una lista de acceso definida para este perfil'],
                400
            );
        }
        $acl->acl = json_decode($acl->acl);
        //return $acl;
        $this->respuesta->data = $acl;
        $this->respuesta->notificacion->mensaje = 'EXITO';
        $this->respuesta->notificacion->tipo = 'success';
        return response()->json($this->respuesta,
            200
        );

    }

    public function store(Peticion $request)
    {
        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);

        $permisos = new ACL;

        $permisos->perfil_id = $request->perfil_id;
        $permisos->usuario_id = $usuarioSistema->data->id;
        $permisos->acl = json_encode($request->acl);
        $permisos->save();

        $this->respuesta->data = ["id"=>$permisos->id];
        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        return response()->json($this->respuesta,
            200
        );

    }

    public function update($id, Peticion $request)
    {


        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);

        $permisos = ACL::find($id);
        $permisos->usuario_id = $usuarioSistema->data->id;
        if ($request->has('perfil_id')) {
            $permisos->rol_id = $request->perfil_id;
        }
        if ($request->has('acl')) {
            $permisos->acl = json_encode($request->acl);
        }else{
            $this->respuesta->notificacion->estado = 1;
            $this->respuesta->notificacion->mensaje = 'Falta informacion para actualizar los permisos';
            $this->respuesta->notificacion->tipo = 'warning';
            return response()->json($this->respuesta,
                417
            );             
        }

        $result = $permisos->save();

        if(!$result){
            
            $this->respuesta->notificacion->estado = -1;
            $this->respuesta->notificacion->mensaje = 'No fue posible actualizar los permisos';
            $this->respuesta->notificacion->tipo = 'success';
            return response()->json($this->respuesta,
                417
            );           
        }
        $this->respuesta->data = ["id"=>$permisos->id];
        $this->respuesta->notificacion->mensaje = 'Permisos actualizados con exito';
        $this->respuesta->notificacion->tipo = 'success';
        return response()->json($this->respuesta,
            200
        ); 
    }

    public function delete($id)
    {
        $permisos = ACL::find($id);
        if(!$permisos){
            $data = new \stdClass;
            $this->respuesta->notificacion->estado = 1;
            $this->respuesta->notificacion->mensaje = 'Registro no encontrado';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                404
            );
        }
        $data = [
            "result" => $permisos->delete(),
        ];
        $this->respuesta->notificacion->mensaje= 'Registro eliminado correctamente';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }
}
