<?php

namespace App\Http\Controllers\Seguridad;

use App\Models\Seguridad\Rol;
use App\Models\Seguridad\ACL;
use App\Http\Controllers\Controller as Controller;
use Illuminate\Http\Request as Peticion;
use Firebase\JWT\JWT;

class RolController extends Controller
{
    
    private $respuesta;
    public function __construct()
    {
        $this->respuesta = new \stdclass();
        $this->respuesta->notificacion = new \stdclass();
        $this->respuesta->notificacion->estado = 0;
        $this->respuesta->notificacion->mensaje = '';
        $this->respuesta->data = new \stdclass();
    }

    public function show(Peticion $request)
    {
        
        $per_page = 15;
        $params = [];
        $sorting = 'nombre';
        $order = 'asc';

        if ($request->has('per_page')) {
            $per_page = $request->per_page;
        }

        if (!is_null($request->getQueryString())) {
            $params = [];
            if ($request->has('id')) {
                $params[] = ['id', '=', $request->input('id')];
            }
            if ($request->has('nombre')) {
                $params[] = ['nombre', 'ilike', "%".$request->input('nombre')."%"];
            }
            $registroEncontrado = Rol::
                where($params)
                ->orderBy($sorting, $order)
                ->paginate($per_page);
            
            if (isset($registroEncontrado)) {
                return $registroEncontrado;
            }
        }
        return Rol::orderBy($sorting, $order)
            ->paginate($per_page);

    }

    public function find($id)
    {
        $data = Rol::find($id);
        $acl = ACL::where(
            [
                ['rol_id', '=', $data->id]
            ]
        )->first();
        if ($acl != NULL) {
            $data->acl = json_decode($acl->acl);
        }else{
            $data->acl = [];
        }

        if(!$data){
            $this->respuesta->notificacion->estado = 1;
            $this->respuesta->notificacion->mensaje = 'Registro no encontrado';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                404
            );
        }

        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );

    }

    public function store(Peticion $request)
    {
        $perfil = new Rol;
        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);

        $perfil->nombre = $request->nombre;
        $perfil->descripcionperfil = $request->descripcion;
        $perfil->usuario_id = $usuarioSistema->data->id;

        $perfil->save();

        return response()->json(["id" => $perfil->id]);
    }

    public function update($id, Peticion $request)
    {
        $token = $request->header('Authorization');
        $usuarioSistema = JWT::decode($token, getenv('APP_KEY'), ['HS256']);

        $perfil = Rol::find($id);

        if ($request->has('nombre')) {
            $perfil->nombre = $request->nombre;
        }
        if ($request->has('descripcion')) {
            $perfil->descripcionperfil = $request->descripcion;
        }
        $perfil->usuario_id = $usuarioSistema->data->id;

        $perfil->save();

        return $perfil;
    }

    public function delete($id)
    {
        $perfil = Rol::findOrFail($id);
        return ["result" => $perfil->delete()];
    }
}
