<?php

namespace App\Http\Controllers\Servicios;

use App\Models\Catalogos\ProductoTelefonica;
use App\Http\Controllers\Controller as Controller;
use Illuminate\Http\Request as Peticion;
use Firebase\JWT\JWT;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;

class ProductoTelefonicaController extends Controller
{
    private $respuesta;
    public function __construct()
    {
        //
        $this->respuesta = new \stdclass();
        $this->respuesta->notificacion = new \stdclass();
        $this->respuesta->notificacion->mensaje = '';
        $this->respuesta->data = new \stdclass();
    }

    public function show(Peticion $request)
    {
        $per_page = 10;
        $params = [];
        $sorting = 'id';
        $order = 'asc';
        if (!is_null($request->getQueryString())) {
            if ($request->has('per_page')) {
                $per_page = $request->per_page;
            }
            if ($request->has('order')) {
                $order = $request->order;
            }
            if ($request->has('sorting')) {
                $sorting = $request->sorting;
            }
            
            $params = [];
            if ($request->has('compania_id')) {
                $params = ['params', '=', $request->compania_id];
            }
            if ($request->has('nombreproducto')) {
                $params[] = ['nombre', 'ilike', "%".$request->nombre."%"];
            }
            if ($request->has('skucompania')) {
                $params[] = ['skucompania', '=', trim($request->skucompania)];
            }
                $data = ProductoTelefonica::where($params)
                ->orderBy($sorting, $order)
                ->paginate($per_page);
                $this->respuesta->notificacion->mensaje = 'Exito';
                $this->respuesta->notificacion->tipo = 'success';
                $this->respuesta->data = $data;
                return response()->json($this->respuesta,
                    200
                );
        }
        $data = ProductoTelefonica::where($params)
                ->orderBy($sorting, $order)
                ->paginate($per_page);

        $this->respuesta->notificacion->mensaje = 'Faltan parámetros';
        $this->respuesta->notificacion->tipo = 'warning';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            400
        );

        

    }

    public function find($id)
    {
        $data = ProductoTelefonica::find($id);
        if(!$data){
            $this->respuesta->notificacion->mensaje = 'No se encontro informacion';
            $this->respuesta->notificacion->tipo = 'warning';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                404
            );
        }
        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function store(Peticion $request)
    {
        $row = new ProductoTelefonica;

        if ($request->has('id')) {
            $row->id = $request->id;
        }
        if ($request->has('compania_id')) {
            $row->compania_id = $request->compania_id;
        }
        if ($request->has('skucompania')) {
            $row->skucompania = $request->skucompania;
        }
        if ($request->has('nombreproducto')) {
            $row->nombreproducto = $request->nombreproducto;
        }
        if ($request->has('montorecarga')) {
            $row->montorecarga = $request->montorecarga;
        }
        if ($request->has('costo')) {
            $row->costo = $request->costo;
        }
        if ($request->has('informacion')) {
            $row->informacion = $request->informacion;
        }
        if ($request->has('stype')) {
            $row->stype = $request->stype;
        }
        if ($request->has('status')) {
            $row->status = $request->status;
        }

        $data = $row->save();

        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );
    }

    public function update($id, Peticion $request)
    {
        
        $row = ProductoTelefonica::find($id);

        if(!$row){
            $this->respuesta->notificacion->mensaje = 'Registro no encontrado';
            $this->respuesta->notificacion->tipo = 'success';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                404
            );
        }
        
        if ($request->has('id')) {
            $row->id = $request->id;
        }
        if ($request->has('compania_id')) {
            $row->compania_id = $request->compania_id;
        }
        if ($request->has('skucompania')) {
            $row->skucompania = $request->skucompania;
        }
        if ($request->has('nombreproducto')) {
            $row->nombreproducto = $request->nombreproducto;
        }
        if ($request->has('montorecarga')) {
            $row->montorecarga = $request->montorecarga;
        }
        if ($request->has('costo')) {
            $row->costo = $request->costo;
        }
        if ($request->has('informacion')) {
            $row->informacion = $request->informacion;
        }
        if ($request->has('stype')) {
            $row->stype = $request->stype;
        }
        if ($request->has('status')) {
            $row->status = $request->status;
        }

        $data = $row->save();

        $this->respuesta->notificacion->mensaje = 'Registro guardado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );


    }

    public function delete($id)
    {
        $row = ProductoTelefonica::find($id);
        if(!$row){
            $this->respuesta->notificacion->mensaje = 'Registro no encontrado';
            $this->respuesta->notificacion->tipo = 'success';
            $this->respuesta->data = $data;
            return response()->json($this->respuesta,
                404
            );
        }

        $result = $row->delete();

        $this->respuesta->notificacion->mensaje = 'Registro eliminado correctamente';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $result;
        return response()->json($this->respuesta,
            200
        );
    }
    public function listaKeyValue(Peticion $request)
    {

        $data = ProductoTelefonica::
        where('status','=',1)
        ->where('compania_id','=', $request->compania_id)
        ->select('id', 'skucompania', 'nombreproducto', 'montorecarga')
        ->orderBy('id', 'asc')
        ->get();

        $this->respuesta->notificacion->mensaje = 'Exito';
        $this->respuesta->notificacion->tipo = 'success';
        $this->respuesta->data = $data;
        return response()->json($this->respuesta,
            200
        );

    }

}
