<?php

/*
|--------------------------------------------------------------------------
| Application Routes
|--------------------------------------------------------------------------
|
| Here is where you can register all of the routes for an application.
| It is a breeze. Simply tell Lumen the URIs it should respond to
| and give it the Closure to call when that URI is requested.
|
*/

$router->get('/', function () use ($router) {
    return $router->app->version();
});

/*
|--------------------------------------------------------------------------
| Rutas para los catalogos
|--------------------------------------------------------------------------
|
| Aquí deberán ir todas la rutas para los catalogos en el servicio
|
*/
$router->group(['namespace' => 'Catalogos'], function() use ($router)
{
    $router->get('/articulos','ArticuloController@show');
    $router->get('/articulos/lista','ArticuloController@listaKeyValue');
    $router->get('/articulos/{id}','ArticuloController@find');
    $router->get('/articulos/offline/catalogo','ArticuloController@getForOffLine');
    $router->post('/articulos','ArticuloController@store');
    $router->put('/articulos/{id}','ArticuloController@update');
    $router->delete('/articulos/{id}','ArticuloController@delete');
    // $router->get('/articulos/item','ArticuloController@getItemVenta');
    


    $router->get('/cajas','CajaController@show');

    $router->get('/departamentos','DepartamentoController@show');
    $router->get('/departamentos/lista','DepartamentoController@listaKeyValue');
    $router->get('/departamentos/{id}','DepartamentoController@find');
    $router->post('/departamentos','DepartamentoController@store');
    $router->put('/departamentos/{id}','DepartamentoController@update');
    $router->delete('/departamentos/{id}','DepartamentoController@delete');

    $router->get('/categorias','CategoriaController@show');
    $router->get('/categorias/lista','CategoriaController@listaKeyValue');
    $router->get('/categorias/{id}','CategoriaController@find');
    $router->post('/categorias','CategoriaController@store');
    $router->put('/categorias/{id}','CategoriaController@update');
    $router->delete('/categorias/{id}','CategoriaController@delete');

    $router->get('/unidadesmedida','UnidadMedidaController@show');
    $router->get('/unidadesmedida/lista','UnidadMedidaController@listaKeyValue');
    $router->get('/unidadesmedida/{id}','UnidadMedidaController@find');
    $router->post('/unidadesmedida','UnidadMedidaController@store');
    $router->put('/unidadesmedida/{id}','UnidadMedidaController@update');
    $router->delete('/unidadesmedida/{id}','UnidadMedidaController@delete');

    $router->get('/clientes','ClienteController@show');
    $router->get('/clientes/lista','ClienteController@listaKeyValue');
    $router->get('/clientes/{id}','ClienteController@find');
    $router->post('/clientes','ClienteController@store');
    $router->put('/clientes/{id}','ClienteController@update');
    $router->delete('/clientes/{id}','ClienteController@delete');

    $router->get('/proveedores','ProveedorController@show');
    $router->get('/proveedores/lista','ProveedorController@listaKeyValue');
    $router->get('/proveedores/{id}','ProveedorController@find');
    $router->post('/proveedores','ProveedorController@store');
    $router->put('/proveedores/{id}','ProveedorController@update');
    $router->delete('/proveedores/{id}','ProveedorController@delete');

    $router->get('/motivosegreso','MotivoEgresoController@show');
    $router->get('/motivosegreso/lista','MotivoEgresoController@listaKeyValue');
    $router->get('/motivosegreso/{id}','MotivoEgresoController@find');
    $router->post('/motivosegreso','MotivoEgresoController@store');
    $router->put('/motivosegreso/{id}','MotivoEgresoController@update');
    $router->delete('/motivosegreso/{id}','MotivoEgresoController@delete');


    $router->get('/formaspago/offline/catalogo','FormaPagoController@getForOffLine');



});

/*
|--------------------------------------------------------------------------
| Rutas para seguridad
|--------------------------------------------------------------------------
|
| Aquí deberán ir todas la rutas que corresponden a seguridad, usuarios
| controles de acceso
|
*/

$router->group(['namespace' => 'Seguridad'], function() use ($router){
   
    /*
    |--------------------------------------------------------------------------
    | Usuarios 
    |--------------------------------------------------------------------------
    */
    $router->get('/usuarios','UsuarioController@show');
    $router->get('/usuarios/lista','UsuarioController@listaKeyValue');
    $router->get('/usuarios/{id}','UsuarioController@find');
    $router->post('/usuarios','UsuarioController@store');
    $router->put('/usuarios/{id}','UsuarioController@update');
    $router->delete('/usuarios/{id}','UsuarioController@delete');


    /*
    |--------------------------------------------------------------------------
    | Roles 
    |--------------------------------------------------------------------------
    */
    $router->get('/roles','RolController@show');
    $router->get('/roles/{id}','RolController@find');
    $router->post('/roles','RolController@store');
    $router->put('/roles/{id}','RolController@update');
    $router->delete('/roles/{id}','RolController@delete');

    /*
    |--------------------------------------------------------------------------
    | Opciones sistema 
    |--------------------------------------------------------------------------
    */

    $router->get('/opcionessistema','OpcionSistemaController@show');
    $router->get('/opcionessistema/menu','OpcionSistemaController@menu');
    $router->get('/opcionessistema/{id}','OpcionSistemaController@find');
    $router->post('/opcionessistema','OpcionSistemaController@store');
    $router->put('/opcionessistema/{id}','OpcionSistemaController@update');
    $router->delete('/opcionessistema/{id}','OpcionSistemaController@delete');
    

    /*
    |--------------------------------------------------------------------------
    | Autenticacion 
    |--------------------------------------------------------------------------
    */
    $router->post('/politicas/login', 'PoliticasController@login');
    $router->get('/politicas/acl/{id}', 'PoliticasController@acl');
    
    /*
    |--------------------------------------------------------------------------
    | ACL
    | Lista de permisos por cada rol
    |--------------------------------------------------------------------------
    */
    $router->get('/acl', 'ACLController@show');
    $router->get('/acl/{id}', 'ACLController@find');
    $router->post('/acl', 'ACLController@store');
    $router->put('/acl/{id}', 'ACLController@update');

    /*

    "token": "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCIsImlzcyI6Imh0dHA6XC9cL3Nub3djbG91ZC5jb20ubXgiLCJhdWQiOiJodHRwOlwvXC9zbm93Y2xvdWQuY29tLm14IiwiaWF0IjoxMzU2OTk5NTI0LCJuYmYiOjEzNTcwMDAwMDAsImRhdGEiOnsibm9tYnJlIjoiSlVBTiIsImFwZWxsaWRvUGF0ZXJubyI6IkNSVVoiLCJhcGVsbGlkb01hdGVybm8iOm51bGwsInBlcmZpbCI6MSwiaWQiOjF9fQ.HE5JncONzqNEXx0tuSHItZBttVJdnZINJsfRDnGcgfY"

    */

    
});

/*
|--------------------------------------------------------------------------
| Rutas para caja
|--------------------------------------------------------------------------
|
| Aquí deberán ir todas la rutas para el modulo de caja
|
*/
$router->group(['namespace' => 'Caja'], function() use ($router)
{
    $router->get('/ventas','VentaController@show');
    $router->get('/ventas/cajeros','VentaController@cajerosActivosCobranza');
    $router->get('/ventas/{id}','VentaController@find');
    $router->post('/ventas','VentaController@store');
    $router->put('/ventas/{id}','VentaController@update');
    $router->delete('/ventas/{id}','VentaController@delete');

    $router->post('/ventas/sync','VentaController@sync');

});

/*
|--------------------------------------------------------------------------
| Rutas para operaciones de caja
|--------------------------------------------------------------------------
|
| Aquí deberán ir todas la rutas para el modulo de operaciones de caja
|
*/
$router->group(['namespace' => 'OperacionesCaja'], function() use ($router)
{

    $router->get('/cortes','CorteCajaController@show');
    $router->get('/cortes/detalle','CorteCajaController@obtenerInfoCorte');
    $router->get('/cortes/verifica','CorteCajaController@verifica');
    $router->get('/cortes/:id','CorteCajaController@find');
    $router->post('/cortes','CorteCajaController@store');
    $router->put('/cortes/:id','CorteCajaController@update');
    $router->delete('/cortes/:id','CorteCajaController@delete');

});

/*
|--------------------------------------------------------------------------
| Rutas para configuracion
|--------------------------------------------------------------------------
|
| Aquí deberán ir todas la rutas para el modulo de caja
|
*/
$router->group(['namespace' => 'Configuracion'], function() use ($router)
{
    $router->get('/configuraciones','ConfiguracionController@show');
    $router->get('/configuraciones/{id}','ConfiguracionController@configurar');
    $router->post('/configuraciones','ConfiguracionController@store');
    $router->put('/configuraciones/{id}','ConfiguracionController@update');
    $router->delete('/configuraciones/{id}','ConfiguracionController@delete');

});

/*
|--------------------------------------------------------------------------
| Rutas para apiserviciospos
|--------------------------------------------------------------------------
|
| Aquí deberán ir todas la rutas que consuman directamente la api para pago
| de servicios
|
*/
$router->group(['namespace' => 'Servicios'], function() use ($router)
{
    //tiempo aire
    $router->get('/telefonicas/productos','ProductoTelefonicaController@show');
    $router->get('/telefonicas/productos/lista','ProductoTelefonicaController@listaKeyValue');
    $router->get('/telefonicas/productos/{id}','ProductoTelefonicaController@find');
    $router->post('/telefonicas/productos','ProductoTelefonicaController@store');
    $router->put('/telefonicas/productos/{id}','ProductoTelefonicaController@update');
    $router->delete('/telefonicas/productos/{id}','ProductoTelefonicaController@delete');

    $router->get('/telefonicas','TelefonicaController@show');
    $router->get('/telefonicas/lista','TelefonicaController@listaKeyValue');
    $router->get('/telefonicas/{id}','TelefonicaController@find');
    $router->post('/telefonicas','TelefonicaController@store');
    $router->put('/telefonicas/{id}','TelefonicaController@update');
    $router->delete('/telefonicas/{id}','TelefonicaController@delete');
    //termina tiempo aire

    $router->get('/tiempoaire/balance','TiempoAireController@getBalance');

});